#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import unittest

from eatb import ROOT
from eatb.metadata.mets import get_package_mets_files_from_basedir, get_mets_obj_id


class METSUtilTest(unittest.TestCase):

    def test_get_package_mets_files_in_directory(self):
        """
        Must return METS.xml files of information package directories contained in a given directory
        """
        mets_files = get_package_mets_files_from_basedir(os.path.join(ROOT, 'tests/test_resources/metadata/mets/'))
        self.assertTrue(len(mets_files) > 0)
        for mets_file in mets_files:
            self.assertTrue(mets_file.endswith("METS.xml"))

    def test_get_identifier_from_mets(self):
        """
        Must return METS.xml files of information package directories contained in a given directory
        """
        mets_file = os.path.join(ROOT, 'tests/test_resources/metadata/mets/METS.xml')
        self.assertEqual("996ed635-3e13-4ee5-8e5b-e9661e1d9a93", get_mets_obj_id(mets_file))


if __name__ == '__main__':
    unittest.main()



from lxml import objectify

from eatb.metadata.parsed_premis import ParsedPremis, P
from eatb.utils.XmlHelper import sequence_insert


class DIPPremis(ParsedPremis):
    """
    DIPPremis class
    """

    def __init__(self, f):
        ParsedPremis.__init__(self, f)

    def get_object_by_identifier(self, identifier = None):
        """
        Find the object with the identifier of the package. If this object does not exist, take the first object of the premis file.
        @param identifier:  Identifier of the package
        @return: premis object element
        """
        if not identifier:
            find = objectify.ObjectPath("premis.object")
            return find(self.root)[0]
        xpath_expr = "//*[local-name() = 'object']/descendant::*[local-name() = 'objectIdentifierValue' and text()='%s']/parent::*/parent::*" % identifier
        xpath_result_nodes = self.root.xpath(xpath_expr)
        object_elm = None if len(xpath_result_nodes) != 1 else xpath_result_nodes[0]
        print(len(xpath_result_nodes))
        print("Type: %s" % type(object_elm))
        return object_elm

    def get_event_identifier_by_name(self, event_name):
        """
        Get the identifier of an event with a given name.
        @param event_name:  Name of the
        @return: event identifier value
        """
        find = objectify.ObjectPath("premis.object")
        xpath_expr = "//*[local-name() = 'eventType' and text()='%s']/parent::*/descendant::*[local-name() = 'eventIdentifierValue']" % event_name
        xpath_result_nodes = self.root.xpath(xpath_expr)
        dip_acquire_aips_id = find(self.root) if len(xpath_result_nodes) != 1 else xpath_result_nodes[0].text
        return dip_acquire_aips_id

    def add_related_aips(self, related_aips, related_event_name, identifier = None):
        """
        Add relationship information of AIPs which were used to create the DIP to the premis object of the package.
        @type       related_aips: list[string]
        @param      related_aips: List of related AIP identifiers
        @type       related_event_name: string
        @param      related_event_name: Name of the related event
        @type       identifier: string
        @param      identifier: Identifier of the object to which the elements are added (first object if none given)
        @rtype:     bool
        @return:    Success/failure of adding the related AIP identifiers
        """
        object_node = self.get_object_by_identifier(identifier)

        find = objectify.ObjectPath("object.objectIdentifier.objectIdentifierValue")
        print(find(object_node).text)

        dip_acquire_aips_id = self.get_event_identifier_by_name(related_event_name)
        sequence_number = 1
        for related_aip in related_aips:
            sequence_insert(
                object_node,
                    P.relationship(
                        P.relationshipType('derivation'),
                        P.relationshipSubType('AIP to DIP conversion'),
                        P.relatedObjectIdentification(
                            P.relatedObjectIdentifierType("repository"),
                            P.relatedObjectIdentifierValue(related_aip),
                            P.relatedObjectSequence(str(sequence_number)),
                        ),
                        P.relatedEventIdentification(
                            P.relatedEventIdentifierType("local"),
                            P.relatedEventIdentifierValue(dip_acquire_aips_id),
                        ),
                    ),

                self.premis_successor_sections
            )
            sequence_number += 1
        return True



import io
import json
import logging
import os
import re

from importlib_resources import files

import lxml.etree as ET


from lxml import isoschematron
from lxml import etree

from eatb.settings import validation_profile
import eatb.resources
from eatb.utils.fileutils import read_file_content
from eatb.utils.xmlutils import XMLSchemaNotFound


class CSIPValidation(object):

    def __init__(self, rules_path=None):
        if not rules_path:
            self.rules_path = files(eatb.resources).joinpath('validation_rules.xml')
        else:
            self.rules_path = rules_path
        xml_rules = self._read_rules_from_location(rules_path)
        self.rules_lines = xml_rules.split('\n')
        self.validation_report = []
        self.validation_profile = json.loads(validation_profile)
        self.valid = True

    @staticmethod
    def _get_rule(lines, rule_id):
        rules_ids = [rule_id]
        n_header_lines = 5
        rule = lines[:n_header_lines]  # add schema and namespaces
        rule_found = False
        # select all patterns with matching id
        for line in lines[n_header_lines:]:
            if not rule_found:
                # look for beginning of pattern/rule
                match = re.search('pattern id="CSIP(.+?)"', line)
                if match:
                    current_id = int(match.group(1))
                    if current_id in rules_ids:
                        # select pattern
                        rule.append(line)
                        rule_found = True
                    # else ignore pattern
                # else is not the beginning of a pattern
            elif '</pattern>' in line:
                # pattern/rule is over
                rule.append(line)
                rule_found = False
            else:
                rule.append(line)
        rule.append('</schema>\n')
        res = '\n'.join(rule)
        return res

    def _process_validation(self, xml_file, rule_id):
        # Parse rules
        single_rule = io.StringIO(CSIPValidation._get_rule(self.rules_lines, rule_id))
        parsed_single_rule = etree.parse(single_rule)
        schematron = isoschematron.Schematron(parsed_single_rule, store_report=True)

        # Parse XML to validate
        parsed_xml_file = etree.parse(xml_file)
        validation_response = schematron.validate(parsed_xml_file)
        report = schematron.validation_report
        return validation_response, report

    def validate(self, ip_path):
        for validation_block_name, validation_rules in self.validation_profile.items():
            validation_block = {validation_block_name: []}
            for rule_id in validation_rules:
                validation_result, report = self._process_validation(ip_path + '/METS.xml', rule_id=rule_id)
                validation_block[validation_block_name].append(
                    {"rule": "csip%d" % rule_id, "result": validation_result, "report": report}
                )
                # set global validation to 'invalid' if one of the rules is violated
                if not validation_result:
                    self.valid = False
            self.validation_report.append(validation_block)
        return self.validation_report

    def get_validation_profile(self):
        return self.validation_profile

    def get_rules_file_path(self):
        return self.rules_path

    def get_log_lines(self):
        log_lines = []
        for validation_blocks in self.validation_report:
            for validation_block_name, validation_results in validation_blocks.items():
                for validation_result in validation_results:
                    if validation_result['result']:
                        log_lines.append({"type": "INFO", "message": "%s - %s - validated successfully" % (
                        validation_block_name, validation_result['rule'])})
                    else:
                        nsmap = {'svrl': 'http://purl.oclc.org/dsdl/svrl'}
                        xml_report = validation_result['report']
                        res = xml_report.find('svrl:failed-assert', namespaces=nsmap)
                        log_lines.append({"type": "ERROR", "message": "%s - %s - Test: %s" % (
                        validation_block_name, validation_result['rule'], res.attrib['test'])})
                        log_lines.append({"type": "ERROR", "message": "%s - %s - Location: %s" % (
                        validation_block_name, validation_result['rule'], res.attrib['location'])})

        return log_lines

    @staticmethod
    def _read_rules_from_location(location):
        if not location:
            return files(eatb.resources).joinpath('validation_rules.xml').read_text()
        with open(location) as fp:
            return fp.read()


class ValidationResult(object):
    def __init__(self, valid, log, err):
        """
        Constructor
        @type       valid: Boolean
        @param      valid: Path to file
        @type       log: List[String]
        @param      log: Processing log
        @type       err: List[String]
        @param      err: Error log
        """
        self.valid = valid
        self.log = log
        self.err = err
    valid = False
    log = []
    err = []


logger = logging.getLogger(__name__)


class XmlValidation(object):
    """
    XML validation class
    """

    XSI = "http://www.w3.org/2001/XMLSchema-instance"
    XS = '{http://www.w3.org/2001/XMLSchema}'

    SCHEMA_TEMPLATE = """<?xml version = "1.0" encoding = "UTF-8"?>
    <schema xmlns="http://www.w3.org/2001/XMLSchema">
    </schema>"""

    def get_schema_from_instance(self, xml):
        xml_dir, tail = os.path.split(xml)

        xml_content = read_file_content(xml)
        tree = ET.XML(xml_content.encode('utf-8'))
        schema_tree = ET.XML(self.SCHEMA_TEMPLATE.encode('utf-8'))

        #Find instances of xsi:schemaLocation
        schema_locations = set(tree.xpath("//*/@xsi:schemaLocation", namespaces={'xsi': self.XSI}))
        for schema_location in schema_locations:
            namespaces_locations = schema_location.strip().split()
            # Import all fnamspace/schema location pairs
            for namespace, location in zip(*[iter(namespaces_locations)] * 2):
                loc = os.path.abspath(os.path.join(xml_dir, location))
                if not os.path.exists(loc):
                    print("ERROR: XML-Schema file not found: %s" % loc)
                    raise XMLSchemaNotFound("XML-Schema file not found: %s" % loc)
                xs_import = ET.Element(self.XS + "import")
                xs_import.attrib['namespace'] = namespace
                xs_import.attrib['schemaLocation'] = loc
                schema_tree.append(xs_import)
        logger.debug("Created schema for validation of instance '%s' from schemaLocation attribute:" % xml)
        logger.debug(ET.tostring(schema_tree, encoding='UTF-8', pretty_print=True, xml_declaration=True))
        return schema_tree

    def validate_XML_by_path(self, xml_path, schema_path):
        """
        This function validates the XML meta data file (parsed_mfile) describing the SIP against the .xsd (sfile),
        which is either contained in the SIP or referenced in the .xml

        @type       xml_path: string
        @param      xml_path: Path to XML file
        @type       schema_path:  string
        @param      schema_path:  Path to schema file.
        @rtype:     ValidationResult
        @return:    Validation result (valid: true/false, processing log, error log)
        """
        validationResult = ValidationResult(False, [], [])
        if schema_path is None:
            validationResult.log.append("XML validation using schemata defined by the schemaLocation attribute.")
        try:
            parsed_schema = ET.parse(schema_path) if schema_path is not None else self.get_schema_from_instance(xml_path)
            parsed_xml = ET.parse(xml_path)
            validationResult = self.validate_XML(parsed_xml, parsed_schema)
        except XMLSchemaNotFound as instance:
            validationResult.err.append("The XML file contains invalid XML-Schema file references. Please verify 'schemaLocation' attribute.")
            validationResult.err.append(instance.parameter)
        except ET.XMLSchemaParseError as xspe:
            # Something wrong with the schema (getting from URL/parsing)
            validationResult.err.append("XMLSchemaParseError occurred!")
            validationResult.err.append(xspe)
        except ET.XMLSyntaxError as xse:
            # XML not well formed
            validationResult.err.append("XMLSyntaxError occurred!")
            validationResult.err.append(xse)
        except Exception as err:
            validationResult.err.append("XML error occurred!")
            validationResult.err.append(err)
        finally:
            return validationResult

    def validate_XML(self, parsed_xml, parsed_schema):
        """
        This function validates the XML meta data file (parsed_mfile) describing the SIP against the .xsd (sfile),
        which is either contained in the SIP or referenced in the .xml

        @type       parsed_xml: ElementTree
        @param      parsed_xml: ElementTree object
        @type       parsed_schema:  ElementTree
        @param      parsed_schema:  Schema file XSD ElementTree object
        @rtype:     bool
        @return:    Validity of XML
        """
        valid_xml = False
        log = []
        err = []
        try:
            # Validate parsed XML against schema returning a readable message on failure
            schema = ET.XMLSchema(parsed_schema)
            # Validate parsed XML against schema returning boolean value indicating success/failure
            log.append('Schema validity: "%s"' % schema.validate(parsed_xml))
            schema.assertValid(parsed_xml)
            valid_xml = True
        except ET.XMLSchemaParseError as xspe:
            # Something wrong with the schema (getting from URL/parsing)
            err.append("XMLSchemaParseError occurred!")
            err.append(xspe)
        except ET.XMLSyntaxError as xse:
            # XML not well formed
            err.append("XMLSyntaxError occurred!")
            err.append(xse)
        except ET.DocumentInvalid as di:
            # XML failed to validate against schema
            err.append("DocumentInvalid occurred!")
            error = schema.error_log.last_error
            if error:
                # All the error properties (from libxml2) describing what went wrong
                err.append('domain_name: ' + error.domain_name)
                err.append('domain: ' + str(error.domain))
                err.append('filename: ' + error.filename)  # '<string>' cos var is a string of xml
                err.append('level: ' + str(error.level))
                err.append('level_name: ' + error.level_name)  # an integer
                err.append('line: ' + str(error.line))  # a unicode string that identifies the line where the error occurred.
                err.append('message: ' + error.message)  # a unicode string that lists the message.
                err.append('type: ' + str(error.type))  # an integer
                err.append('type_name: ' + error.type_name)
        finally:
            return ValidationResult(valid_xml, log, err)



""" Root E-ARK Toolbox module. """
#!/usr/bin/env python
# -*- coding: utf-8 -*-
import logging
import os
from logging.config import fileConfig

ROOT = os.path.split(os.path.abspath(os.path.dirname(__file__)))[0]

fileConfig(os.path.join(ROOT, 'eatb/logging_config.ini'))
LOGGER = logging.getLogger()

logger = logging.getLogger(__name__)

VersionDirFormat = 'v%05d'



#!/usr/bin/env python
# coding=UTF-8
import configparser
import string
import subprocess

import os

from eatb import ROOT

class CliCommand(object):
    """
    CliCommand class
    Get a Command Line Interface (CLI) command based on templates defined in the configuration.
    """
    def __init__(self, name, cmd_template):
        """
        Create a command
        :param name: Command name
        :param cmd_template: Command template
        """
        self.name = name
        self.cmd_template = cmd_template

    def get_command(self, subst):
        """
        Get a Command Line Interface (CLI) command based on templates defined in the configuration.
        :param subst: Dictionary of substitution variables (e.g. {'foo': 'bar_v00000_b00001'} where 'foo' is the variable and 'bar_v00000_b00001'
        the substitution value).
        :return: list of command elements
        """
        res_cmd = []
        for cmd_part in self.cmd_template:
            if isinstance(cmd_part, string.Template):
                res_cmd.append(cmd_part.substitute(subst))
            else:
                res_cmd.append(cmd_part)
        return res_cmd


class CliExecution(object):
    """
    CliCommand class
    Get a Command Line Interface (CLI) command based on templates defined in the configuration.
    """
    def __init__(self, command):
        """
        Create a command
        :param command: Command
        """
        assert type(command) is list, "command must be of type list"
        self.command = command

    def execute(self):
        return subprocess.check_output(self.command)


class CliCommands(object):
    """
    CliCommands class
    List of CLI commands based on configuration file.
    """
    def __init__(self, commands_config_file=None):
        """
        Create commands list based on configuration file.
        :param commands_config_file: Configuration file
        """
        self.config = configparser.ConfigParser()
        self.config['DEFAULT'] = {}
        self.config.sections()
        if not commands_config_file:
            commands_config_file = os.path.join(ROOT, 'eatb/resources/default_commands.cfg')
        if not os.path.exists(commands_config_file):
            raise FileNotFoundError("Configuration not found: %s (root: %s)" % (commands_config_file, ROOT))
        self.config.read(commands_config_file)

    def get_command_template(self, command_id):
        """
        Get a Command Line Interface (CLI) command based on templates defined in the configuration.
        :param command_id: Identifier of the command
        :return: command list
        """
        command_template = self.config.get('commands', command_id)

        cmd_parts = command_template.split()

        def prep_cmd_part(cmd_part):
            if cmd_part.startswith("string.Template"):
                return eval(cmd_part)
            return cmd_part

        return [prep_cmd_part(cmd_part) for cmd_part in cmd_parts]



import json
import os
import re
from shutil import copytree
from shutil import rmtree
import fnmatch
import shutil
import mimetypes
from urllib.parse import quote, unquote

from eatb.utils.datetime import get_file_ctime_iso_date_str, EU_UI_FORMAT, get_local_datetime_now
from eatb.utils.reporters import default_reporter

MAX_TRIES = 10000
mimetypes.add_type('text/plain', '.log')


def remove_dir(path):
    """
    Delete directory and contents (tree)
    :param path: path
    :return: success deleting directory
    """
    if os.path.isdir(path):
        rmtree(path)
        return not os.path.exists(path)


def remove_fs_item(uuid, working_dir, rel_path):
    """
    Remove file system item only if it is in a specific working directory
    :param uuid: UUID of the working direcotry
    :param working_dir: working directory
    :param rel_path: relative path within the working directory of the item to be removed
    :return: success deleting file system item
    """
    abs_path = os.path.join(working_dir, rel_path)
    if working_dir.endswith(uuid) and os.path.exists(abs_path):
        if os.path.isdir(abs_path):
            rmtree(abs_path)
        else:
            os.remove(abs_path)
    return not os.path.exists(abs_path)


def remove_protocol(path_without_protocol):
    """
    Remove protocol from path
    :param path_without_protocol:
    :return: path without protocol
    """
    protocols = ['file://', 'file:', 'file/']
    for prot in protocols:
        stripped_path = path_without_protocol.replace(prot, '')
        if len(stripped_path) < len(path_without_protocol):
            return stripped_path
    return path_without_protocol


def copy_tree_content(source_dir, target_dir):
    """
    Copy content of a directory as tree
    :param source_dir: source directory
    :param target_dir: target directory
    :return: None
    """
    fs_childs = os.listdir(source_dir)
    for fs_child in fs_childs:
        source_item = os.path.join(source_dir, fs_child)
        copytree(source_item, os.path.join(target_dir, fs_child))


def move_folder_content(source_dir, target_dir):
    """
    Move the content of a folder
    :param source_dir: source directory
    :param target_dir: target directory
    :return: None
    """
    fs_childs = os.listdir(source_dir)
    for fs_child in fs_childs:
        source_item = os.path.join(source_dir, fs_child)
        shutil.move(source_item, target_dir)


def force_merge_flat_dir(source_dir, target_dir):
    """
    Merge directories with flat structure
    :param source_dir: source directory
    :param target_dir: target directory
    :return: None
    """
    if not os.path.exists(target_dir):
        os.makedirs(target_dir)
    for item in os.listdir(source_dir):
        source_file = os.path.join(source_dir, item)
        destination_file = os.path.join(target_dir, item)
        secure_force_copy_file(source_file, destination_file)


def secure_force_copy_file(source_file, target_file):
    """
    Secure file copy (checksum test)
    :param source_file:
    :param target_file:
    :return: True if target file exists and checksum verification was successful
    :raises: IOError if the checksum verification failed
    """
    if os.path.isfile(source_file):
        shutil.copy2(source_file, target_file)
        from eatb.checksum import get_sha256_hash
        if not os.path.exists(target_file) or not get_sha256_hash(source_file) == get_sha256_hash(target_file):
            raise IOError("File copy operation failed  (checksums not equal: %s vs. %s)." % (source_file, target_file))
        return os.path.exists(target_file)


def is_flat_dir(source_directory):
    """
    Check if a directory has a flat structure
    :param source_directory: Source directory
    :return: is flat directory (bool)
    """
    for item in os.listdir(source_directory):
        source_item = os.path.join(source_directory, item)
        if os.path.isdir(source_item):
            return False
    return True


def secure_copy_tree(source_file, target_file):
    """
    Secure tree copy (checksum test)
    :param source_file:
    :param target_file:
    :return: success of copying the content (bool)
    """
    for item in os.listdir(source_file):
        s = os.path.join(source_file, item)
        d = os.path.join(target_file, item)
        if os.path.isfile(s):
            if not os.path.exists(target_file):
                os.makedirs(target_file)
            secure_force_copy_file(s, d)
        if os.path.isdir(s):
            is_recursive = not is_flat_dir(s)
            if is_recursive:
                secure_copy_tree(s, d)
            else:
                force_merge_flat_dir(s, d)
    return os.path.exists(target_file)


def delete_directory_content(root_dir):
    """
    Delete content of a directory (root directory remains)
    :param root_dir:
    :return: success deleting directory content
    """
    fs_childs = os.listdir(root_dir)
    for fs_child in fs_childs:
        item = os.path.join(root_dir, fs_child)
        if os.path.isdir(item):
            shutil.rmtree(item)
        else:
            os.remove(item)
    return len(os.listdir(root_dir)) == 0


def increment_file_name_suffix(abspath_basename, extension):
    """
    Increment file name suffix
    :param abspath_basename: absolute path without extension
    :param extension: file extension
    :return: incremented file path
    """
    i = 1
    while i < MAX_TRIES:
        suffix = '%05d' % i
        inc_file_name = "%s_%s.%s" % (abspath_basename, suffix, extension)
        if not os.path.exists(inc_file_name):
            return inc_file_name
        i += 1


def latest_file_by_suffix(abspath_basename, extension):
    """
    Get latest file by suffix
    :param abspath_basename: absolute path without extension
    :param extension: file extension
    :return: incremented file path
    """
    i = 1
    file_candidate = None
    while i < MAX_TRIES:
        suffix = '%05d' % i
        inc_file_name = "%s_%s.%s" % (abspath_basename, suffix, extension)
        if not os.path.exists(inc_file_name):
            return file_candidate
        file_candidate = inc_file_name
        i += 1


def read_and_load_json_file(file_path):
    info_file_content = read_file_content(file_path)
    return json.loads(info_file_content)


def read_file_content(file_path):
    """
    Read content of a file
    :param file_path: file path
    :return: file content
    """
    mime = get_mime_type(file_path)
    mode = "r" if mime and (mime.startswith("text") or mime.endswith("json") or mime.endswith("xml")) else "rb"
    fh = open(file_path, mode)
    file_content = fh.read()
    return file_content


def locate(pattern, root_path):
    """
    Find a file within a root path
    :param pattern: Pattern (fnmatch)
    :param root_path: Root path
    :return: Generator of file list
    """
    for path, dirs, files in os.walk(os.path.abspath(root_path)):
        for filename in fnmatch.filter(files, pattern):
            yield os.path.join(path, filename)


def find_files(directory, pattern):
    """
    Find files in directory (generator)
    :param directory: directory
    :param pattern: pattern (fnmatch)
    :return: Generator for list of files
    """
    for root, dirs, files in os.walk(directory):
        for basename in files:
            if fnmatch.fnmatch(basename, pattern):
                filename = os.path.join(root, basename)
                yield filename


def copy_file_with_base_directory(source_root, target_root, sub_path, source_file_name):
    """
    Copy file with base directory
    :param source_root: source root directory
    :param target_root: target root directory
    :param sub_path: sub-path
    :param source_file_name: source file name
    :return: success of copying the file (bool)
    """
    rel_sub_path = sub_path.lstrip("/")
    target_dir = os.path.join(target_root, rel_sub_path)
    os.makedirs(target_dir, exist_ok=True)
    source_file_path = os.path.join(source_root, rel_sub_path, source_file_name)
    target_file_path = os.path.join(target_root, rel_sub_path, source_file_name)
    shutil.copy2(source_file_path, target_file_path)
    return os.path.exists(target_file_path)


def copy_folder(source_directory, destination_directory):
    """
    Copy folder
    :param source_directory: source directory
    :param destination_directory: destination directory
    :return: success of copying directory (bool)
    """
    _, leaf_folder = os.path.split(source_directory)
    shutil.copytree(source_directory, os.path.join(destination_directory, leaf_folder))
    return os.path.exists(destination_directory)


def get_immediate_subdirectories(directory):
    """
    Get a list of sub-directories of a directory
    :param directory:
    :return: list of sub-directories
    """
    return [name for name in os.listdir(directory)
            if os.path.isdir(os.path.join(directory, name))]


def list_files_in_dir(directory):
    """
    List files in directory
    :param directory:
    :return:
    """
    return [name for name in os.listdir(directory)
            if os.path.isfile(os.path.join(directory, name))]


def sub_dirs(directory):
    """
    Get a list of sub-directories of a directory
    :param directory:
    :return: list of sub-directories
    """
    return sorted([name for name in os.listdir(directory) if os.path.isdir(os.path.join(directory, name))])


def delete_from_dir(directory, pattern):
    """
    Delete files from a directory
    :param directory: directory
    :param pattern: pattern
    :return:
    """
    for f in os.listdir(directory):
        if re.search(pattern, f):
            path = os.path.join(directory, f)
            os.remove(path)
            if os.path.exists(path):
                return False
    return True


def purge(directory, pattern):
    """
    Delete all files from a directory
    :param directory: directory
    :param pattern: pattern
    :return:
    """
    for f in os.listdir(directory):
        if re.search(pattern, f):
            os.remove(os.path.join(directory, f))
    return len(os.listdir(directory)) == 0


def rec_find_files(directory, include_files_rgxs=None, exclude_dirsfiles_rgxs=None):
    """
    Find files matching pattern
    :param directory: directory where recursive search starts
    :param include_files_rgxs: regex patterns of files to be included (filter)
    :param exclude_dirsfiles_rgxs: regex patterns of directories or files to be excluded
    :return: Generator of matching files list
    """
    assert not include_files_rgxs or type(include_files_rgxs) is list, \
        "param 'includes_files' must be of type list"
    assert not exclude_dirsfiles_rgxs or type(exclude_dirsfiles_rgxs) is list, \
        "param 'excludes_dirs_files' must be of type list"
    includes_pattern = r'|'.join([x for x in include_files_rgxs]) if include_files_rgxs else None
    excludes_pattern = r'|'.join([x for x in exclude_dirsfiles_rgxs]) if exclude_dirsfiles_rgxs else None
    includes_rgxs = re.compile(includes_pattern) if includes_pattern else None
    excludes_rgxs = re.compile(excludes_pattern) if excludes_pattern else None
    for root, dirs, files in os.walk(directory):
        if not excludes_rgxs or not re.match(excludes_rgxs, root):
            if not excludes_rgxs or not re.match(excludes_rgxs, root):
                for f in files:
                    if not excludes_rgxs or not re.match(excludes_rgxs, f):
                        if not include_files_rgxs or re.match(includes_rgxs, f):
                            filename = os.path.join(root, f)
                            yield filename


def path_to_dict(path, strip_path_part=None, use_icons=False):
    d = {'text': os.path.basename(path)}
    if os.path.isdir(path.encode('utf-8')):
        if use_icons:
            d['icon'] = "glyphicon glyphicon-folder-close"
        d['children'] = [path_to_dict(os.path.join(path, x), strip_path_part) for x in sorted(os.listdir(path))]
        path_metadata = path if strip_path_part is None else path.replace(strip_path_part, "")
        d['data'] = {"path": path_metadata}
    else:
        if use_icons:
            d['icon'] = "glyphicon glyphicon-file"
        path_metadata = path if strip_path_part is None else path.replace(strip_path_part, "")
        wd_path, _ = os.path.split(path)
        mimetype = get_mime_type(path)
        mimetype = mimetype if mimetype else "application/octet-stream"
        d['data'] = {"path": path_metadata, "mimetype": mimetype,
                     "datetime": get_file_ctime_iso_date_str(path, EU_UI_FORMAT, '/var/data/earkweb')}
    return d


def total_directory_size(start_path='.'):
    """
    Get the total file size of directory and subdirectories
    :param start_path: file path
    :return: total file size of directory and subdirectories in bytes (integer value)
    """
    total_size = 0
    for dirpath, dirnames, filenames in os.walk(start_path):
        for f in filenames:
            fp = os.path.join(dirpath, f)
            # skip if it is symbolic link
            if not os.path.islink(fp):
                total_size += os.path.getsize(fp)

    return total_size


def fsize(file_path, wd=None):
    """
    Get the file size of a file in the file system
    :param file_path: file path
    :param wd: working directory
    :return: file size in bytes (integer value)
    """
    fp = file_path
    path = fp if wd is None else os.path.join(wd, fp)
    return int(os.path.getsize(path))


def human_readable_size(size):
    """
    Get a fhuman readable string representation for a given file size in bytes (integer)
    :param size: size (integer)
    :return: human readable string representation of the file size
    """
    for unit in ['', 'Ki', 'Mi', 'Gi', 'Ti', 'Pi', 'Ei', 'Zi']:
        if abs(size) < 1024.0:
            return "%3.1f%s" % (size, unit)
        size /= 1024.0


def get_mime_type(path):
    """
    Get mime type of a file
    :param path: path to file
    :return: mime type (tuple: (type, subtype))
    """
    mime_type, subtype = mimetypes.guess_type(path)
    mime_type = mime_type if mime_type else "application/octet-stream"
    return mime_type


def get_directory_json(path, subpath):
    """
    Get directory JSON representation
    :param path: path
    :param subpath: sub-path
    :return: JSON representation of directory
    """
    joined_path = os.path.join(path, subpath)
    dir_json = path_to_dict(joined_path, strip_path_part=path + '/')
    return {"data": dir_json, "check_callback": "true"}


def strip_prefixes(path, *prefixes):
    """
    Strip prefixes
    :param path: path
    :param prefixes: prefixes
    :return: String without prefixes
    """
    prefix_part = os.path.join(os.path.join(*prefixes), '')
    return path.replace(prefix_part, '')


def backup_file_path(abs_path):
    """
    Backup file path
    :param abs_path: absolute path
    :return: return path for backup
    """
    path, filename = os.path.split(abs_path)
    basename, extension = os.path.splitext(filename)
    dt = get_local_datetime_now()
    ts = dt.strftime("%Y%m%d%H%M%S")
    backup_file_name = "%s_%s%s" % (basename, ts, extension)
    return os.path.join(path, backup_file_name)


def package_sub_path_from_relative_path(root, containing_file_path, relative_path):
    """
    Get package sub-path from relative path
    param root: root path
    param containing_file_path: containing file path
    param relative_path: relative path
    :return: rpackage sub-path
    """
    containing_path, _ = os.path.split(containing_file_path)
    return strip_prefixes(os.path.abspath(os.path.join(containing_path, remove_protocol(relative_path))), root)


def get_sub_path_from_relative_path(root, containing_file_path, relative_path):
    """
    Get sub-path from relative path
    :param root: root directory
    :param containing_file_path: containing file path
    :param relative_path: relative path
    :return: sub-path
    """
    containing_path, _ = os.path.split(containing_file_path)
    return strip_prefixes(os.path.abspath(os.path.join(containing_path, relative_path)), root)

def from_safe_filename(cleaned_identifier):
    """
    Get original identifier from safe filename identifier string, see
    https://www.ietf.org/archive/id/draft-kunze-pairtree-01.txt
    :param cleaned_identifier: file name identifier string (https=+doi,org=10,5281=zenodo,4514864)
    :return: original identifier (https://doi.org/10.5281/zenodo.4514864)
    """
    # Reverse second step: single-character to single-character conversions
    translation_table = str.maketrans("=+,", "/:.")
    intermediate_identifier = cleaned_identifier.translate(translation_table)
    # Reverse first step: hexadecimal decoding
    def hex_decode(match):
        hex_value = match.group(1)
        return chr(int(hex_value, 16))
    # Replace hexadecimal encoded characters with their original characters
    original_identifier = re.sub(r'\^([0-9a-fA-F]{2})', hex_decode, intermediate_identifier)
    return original_identifier

def to_safe_filename(identifier):
    """
    Get safe filename identifier string from original identifier, see
    https://www.ietf.org/archive/id/draft-kunze-pairtree-01.txt
    :param identifier: identifier (https://doi.org/10.5281/zenodo.4514864
    :return: file name identifier string (https=+doi,org=10,5281=zenodo,4514864)
    """
    # First step: Hexadecimal encoding for special characters
    special_chars = {
        '"': '^22', '*': '^2a', '+': '^2b', ',': '^2c',
        '<': '^3c', '=': '^3d', '>': '^3e', '?': '^3f',
        '\\': '^5c', '^': '^5e', '|': '^7c', ' ': '^20'
    }
    
    def hex_encode(char):
        if char in special_chars:
            return special_chars[char]
        elif ord(char) < 0x21 or ord(char) > 0x7e:
            return f'^{ord(char):02x}'
        else:
            return char
            
    # apply conversion
    safe_identifier = ''.join(hex_encode(c) for c in identifier)
    
    # Second step: Single-character to single-character conversions
    translation_table = str.maketrans("/:.", "=+,")
    mapped_identifier = safe_identifier.translate(translation_table)
    return mapped_identifier

def encode_identifier(identifier: str) -> str:
    """
    Encode an identifier string for safe inclusion in a URL.

    :param identifier: The identifier string to be encoded.
    :return: The encoded identifier string.
    """
    return quote(to_safe_filename(identifier))


def decode_identifier(encoded_identifier: str) -> str:
    """
    Decode an identifier string for safe inclusion in a URL.

    :param encoded_identifier: The encoded identifier string to be decoded.
    :return: The decoded identifier string.
    """
    return from_safe_filename(unquote(encoded_identifier))

class FileBinaryDataChunks(object):
    """
    Generator to iteratively read binary chunks of a large file
    """

    def __init__(self, filepath, chunksize=65536, progress_reporter=default_reporter):
        """
        Initialise object with file path and chunksize
        :param filepath: File path
        :param chunksize: Chunk size
        """
        self.filepath = filepath
        self.chunksize = chunksize
        self.progress_reporter = progress_reporter
        self.current_file_size = os.path.getsize(filepath)
        self.bytesread = 0

    def chunks(self, total_bytes_read=0, bytes_total=-1):
        """
        Chunk generator, returns data chunks which can be iterated in a for loop.
        :return: generator with data chunks
        """
        if bytes_total == -1:
            bytes_total = self.current_file_size
        f = open(self.filepath, 'rb')

        def readchunk():
            return f.read(self.chunksize)

        for chunk in iter(readchunk, ''):
            self.bytesread += len(chunk)
            percent = (total_bytes_read+self.bytesread) * 100 / bytes_total
            self.progress_reporter(percent)
            yield chunk


def copy_file(source, target, progress_reporter=default_reporter, total_bytes_read=0):
    with open(target, 'wb') as target_file:
        for chunk in FileBinaryDataChunks(source, 65536, progress_reporter).chunks(total_bytes_read, total_bytes_read):
            target_file.write(chunk)
        target_file.close()
        total_bytes_read += fsize(source)
    return total_bytes_read



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from eatb.metadata.parsed_ead import ParsedEad
from eatb.utils.datetime import LengthBasedDateFormat


def field_namevalue_pairs_per_file(extract_defs, ead_root_path, ead_file_path):
    eadparser = ParsedEad(ead_root_path, ead_file_path)
    file_elmvalpairs = dict()
    for ed in extract_defs:
        text_access_path = ed['text_access_path'] if 'text_access_path' in ed else None
        is_attribute = ed['is_attribute'] if 'is_attribute' in ed else None
        file_elmvalpairs[ed['solr_field']] = eadparser.dao_path_mdval_tuples(ed['ead_element'], text_access_path, is_attribute)
    result = dict()
    for element_name, file_value_pair_list in file_elmvalpairs.items():
        for file_value_pair in file_value_pair_list:
            if not file_value_pair['path'] in result.keys():
                result[file_value_pair['path']] = []

            reformatted_md_value = file_value_pair['mdvalue']
            if element_name.endswith("_dt"):
                lbdf = LengthBasedDateFormat(file_value_pair['mdvalue'])
                print("DATE: %s" % file_value_pair['mdvalue'])
                reformatted_md_value = lbdf.reformat()
            result[file_value_pair['path']].append({'field_name': element_name, 'field_value': reformatted_md_value})
    return result





#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import generators

import logging
import os
import re
import sys
import tarfile
import zipfile
from abc import ABC, abstractmethod
from pathlib import Path
from subprocess import check_output
from eatb.cli import CliCommands, CliCommand
from eatb.checksum import ChecksumFile, ChecksumAlgorithm
from eatb.utils.reporters import default_reporter


class ManifestCreation():
    """
    Create package file manifest using 'summain'
    """
    def __init__(self, working_directory, commands=CliCommands()):
        self.working_directory = working_directory
        self.commands = commands
        if not os.path.exists(working_directory):
            os.makedirs(working_directory)

    def create_manifest(self, package_dir, manifest_file):
        cli_command = CliCommand("summain", self.commands.get_command_template("summain"))
        command = cli_command.get_command(dict(manifest_file=manifest_file, package_dir=package_dir))
        out = check_output(command)
        return out


def create_package(input_directory, packagename, gunzip=False, output_directory=None, use_input_dir_as_root=False, exclude=[]) -> str:
    """
    Create package
    :param input_directory: Input directory
    :param packagename: Name of the package file
    :param output_directory: Output directory (same as input directory if not provided)
    :gunzip create compressed container package
    :return: md5checksum of package
    """

    # append generation number to tar file; if tar file exists, the generation number is incremented
    input_path = Path(input_directory)
    extension = '.tar.gz' if gunzip else '.tar'
    if not output_directory:
        output_directory = input_path.parent
    storage_tar_file = os.path.join(output_directory, packagename + extension)
    write_mode = "w:gz" if gunzip else "w:"
    tar = tarfile.open(storage_tar_file, "w:")
    total = sum([len(files) for (root, dirs, files) in os.walk(input_directory)])
    i = 0
    for subdir, dirs, files in os.walk(input_directory):

        for dir in dirs:
            entry = os.path.join(os.path.basename(input_directory), subdir, dir)
            base = os.path.basename(input_directory) if use_input_dir_as_root else ""
            if not os.listdir(entry):
                tar.add(entry, arcname=os.path.join(base, os.path.relpath(entry, input_directory)))

        for file in files:
            entry = os.path.join(os.path.basename(input_directory), subdir, file)
            base = os.path.basename(input_directory) if use_input_dir_as_root else ""

            def exclude_func(file_name):
                return os.path.basename(file_name) in exclude

            tar.add(entry, arcname=os.path.join(base, os.path.relpath(entry, input_directory)))
            if i % 10 == 0:
                perc = (i * 100) / total
            i += 1
    tar.close()

    checksum_file = ChecksumFile(storage_tar_file)
    return checksum_file.get(ChecksumAlgorithm.MD5)


class PackageFormat:
    TARGZ, TAR, ZIP, NONE = range(4)

    @staticmethod
    def get(filename):
        """
        Constructor
        :param filename: filename
        :return:  Package Format
        """
        if filename.endswith("tar.gz"):
            return PackageFormat.TARGZ
        if filename.endswith("tar"):
            return PackageFormat.TAR
        if filename.endswith("zip"):
            return PackageFormat.ZIP
        return PackageFormat.NONE

    @staticmethod
    def str(alg):
        if alg is PackageFormat.TARGZ:
            return "TARGZ"
        if alg is PackageFormat.TAR:
            return "TAR"
        if alg is PackageFormat.ZIP:
            return "ZIP"
        return "NONE"


logger = logging.getLogger(__name__)


class PackagedContainer(ABC):

    def __init__(self, container_file, stream=None):
        self.container_file = container_file
        if stream is None:
            self.stream = stream if stream else sys.stdout
        self.success = False
        self.percent = 0
        self.number_of_items = 0

    @abstractmethod
    def extract(self, extract_to):
        pass

    @staticmethod
    def factory(filename):
        if PackageFormat.get(filename) == PackageFormat.TAR or PackageFormat.get(filename) == PackageFormat.TARGZ:
            return TarContainer(filename)
        if PackageFormat.get(filename) == PackageFormat.ZIP:
            return ZipContainer(filename)
        assert 0, "Package format not supported"


class TarContainer(PackagedContainer):
    """
    Extract TAR container
    """

    def __init__(self, container_file, stream=None):
        super().__init__(container_file, stream)

    def extract(self, extract_to):
        """
        This function extracts a SIP file to a target directory.
        :param package_file_path: package file path
        :param extract_to: target directory
        :return:
        """
        total = 0
        try:
            tar_object = tarfile.open(name=self.container_file, mode='r', encoding='utf-8')
            members = tar_object.getmembers()
            total = len(members)
            print("Total: " + str(total))
            i = 100
            for member in members:
                if i % 100 == 0:
                    perc = (i*100)/total
                    logger.debug("100 processed (item %d) ... (%d)" % (i, perc))
                tar_object.extract(member, extract_to)
                i += 1
        except (ValueError, OSError, IOError, tarfile.TarError) as why:
            logger.error('Problem to extract %s' % self.container_file, why)
        return total

    def has_member(self, package_file_path, member_name):
        tar_object = tarfile.open(name=package_file_path, mode='r', encoding='utf-8')
        try:
            tar_object.getmember(member_name)
            return True
        except KeyError:
            return False

    def extract_with_report(self, package_file_path, extract_to, progress_reporter=default_reporter,
                            total=0, current=0):
        try:
            logger.info("Extracting package %s to %s" % (package_file_path, extract_to))
            tar_object = tarfile.open(name=package_file_path, mode='r', encoding='utf-8')
            members = tar_object.getmembers()
            total = len(members)
            for member in members:
                if current % 2 == 0:
                    perc = (current * 100) / total
                    progress_reporter(perc)
                tar_object.extract(member, extract_to)
                current += 1
        except (ValueError, OSError, IOError, tarfile.TarError) as why:
            logger.error('Problem to extract %s' % package_file_path, why)
        return total


class ZipContainer(PackagedContainer):

    def __init__(self, container_file, stream=None):
        super().__init__(container_file, stream)

    def extract(self, extract_to):
        return self.unzip(extract_to)

    def unzip(self, directory, progress_reporter=default_reporter):
        self.percent = 10
        try:
            if not directory.endswith(':') and not os.path.exists(directory):
                os.mkdir(directory)

            zf = zipfile.ZipFile(self.container_file)

            # create directory structure
            self._createstructure(self.container_file, directory)

            num_files = len(zf.namelist())
            self.number_of_items = num_files

            self.stream.write("Extracting %s items (directories and files)\n" % num_files)

            percent = self.percent
            divisions = 100 / percent

            percent = int(num_files / divisions)

            # extract files to directory structure
            for i, name in enumerate(zf.namelist()):
                if percent > 0 and (i % percent) == 0 and i > 0:
                    complete = int(i / percent) * percent
                    progress_reporter(complete)

                if not name.endswith('/'):
                    outfile = open(os.path.join(directory, name), 'wb')
                    outfile.write(zf.read(name))
                    outfile.flush()
                    outfile.close()
            num_f = sum([len(files) for r, d, files in os.walk(directory)])
            self.stream.write("Extracted %s files\n" % num_f)
            self.success = True
        except ValueError as e:
            self.stream.write('Problem to extract %s: %s\n' % (self.container_file, str(e)))
            self.success = False
        except OSError as e:
            self.stream.write('Problem to extract %s: %s\n' % (self.container_file, str(e)))
            self.success = False
        except IOError as e:
            self.stream.write('Problem to extract %s: %s\n' % (self.container_file, str(e)))
            self.success = False
        return self.success

    @staticmethod
    def has_member(package_file_path, member_name):
        zf = zipfile.ZipFile(package_file_path)
        return member_name in zf.namelist()

    def _createstructure(self, file, directory):
        self._makedirs(self._listdirs(file), directory)

    @classmethod
    def _makedirs(cls, directories, basedir):
        """ Create directories """
        for directory in directories:
            curdir = os.path.join(basedir, directory)
            if not os.path.exists(curdir):
                os.makedirs(curdir)
                print(curdir)

    @classmethod
    def _listdirs(cls, file):
        """ Get the directories """
        zf = zipfile.ZipFile(file)
        dirs = []
        filelist = filter(lambda x: not x.endswith('/'), zf.namelist())
        dirlist = filter(lambda x: x.endswith('/'), zf.namelist())
        for f in filelist:
            path, filename = os.path.split(f)
            if path.startswith("/"):
                path = path[1:len(path)]
            dirs.append(path)
        dirs.extend(dirlist)
        dirs.sort()
        return dirs


class ChunkedTarEntryReader(object):
    """
    Chunked TAR entry reader allowing to read large TAR entries.
    """

    def __init__(self, tfile: tarfile, chunksize: int = 512):
        self.tfile = tfile
        self.chunksize = chunksize
        self.bytesread = 0

    def close(self):
        if self.tfile:
            return self.tfile.close()
        return True

    def chunks(self, entry, total_bytes_read=0, bytes_total=-1):
        """
        Chunk generator, returns data chunks which can be iterated in a for loop.
        :return: generator with data chunks
        """
        tinfo = self.tfile.getmember(entry)

        if bytes_total == -1:
            bytes_total = tinfo.size
        f = self.tfile.extractfile(tinfo)

        def readchunk():
            return f.read(self.chunksize)

        for chunk in iter(readchunk, ''):
            self.bytesread += len(chunk)
            percent = (total_bytes_read+self.bytesread) * 100 / bytes_total
            default_reporter(percent)
            yield chunk
            if percent == 100:
                break

def get_deliveries(path, task_logger):
    package_files = [f for f in os.listdir(path) if re.search(r'.*\.(zip|tar)$', f)]
    task_logger.addinfo("Package files found: %s" % package_files)
    deliveries = {}
    for package_file in package_files:
        tar_base_name, _ = os.path.splitext(package_file)
        delivery_file = os.path.join(path, ("%s.xml" % tar_base_name))
        task_logger.addinfo("Looking for delivery file: %s" % delivery_file)
        if os.path.exists(delivery_file):
            deliveries[tar_base_name] = {"delivery_xml": delivery_file, "package_file": package_file}
        else:
            task_logger.addinfo("Warning: no delivery file found for package: %s" % tar_base_name)
    return deliveries



import re
import unittest


def lstrip_substring(instr, substr):
    """
    Remove substring from string at the beginning
    :param instr: original string
    :param substr: substring
    :return: result string
    """
    if instr.startswith(substr):
        len_substr = len(substr)
        return instr[len_substr:]
    else:
        return instr


def safe_path_string(value):
    """
    Normalizes string, converts to lowercase, removes non-alpha characters,
    and converts spaces to underscore.
    """
    import unicodedata
    import re
    value = unicodedata.normalize('NFKD', value).encode('ascii', 'ignore')
    value = re.sub('[^\w\s-]', '', value).strip().lower()
    return re.sub('[-\s]+', '_', value)


def multiple_replacer(*key_values):
    import re
    replace_dict = dict(key_values)
    replacement_function = lambda match: replace_dict[match.group(0)]
    pattern = re.compile("|".join([re.escape(k) for k, v in key_values]), re.M)
    return lambda string: pattern.sub(replacement_function, string)


def multiple_replace(string, *key_values):
    return multiple_replacer(*key_values)(string)


def whitespace_separated_text_to_dict(textmap):
    result_map = {}
    for line in textmap.splitlines():
        line = line.strip()
        if line:
            columns = line.rsplit(' ', 1)
            entry = {columns[0].strip(): columns[1].strip()}
            result_map.update(entry)
    return result_map



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import shutil
import unittest
import re
import io

from lxml import isoschematron
from lxml import etree

from eatb import ROOT
from eatb.utils.fileutils import rec_find_files
from eatb.utils.randomutils import randomword


def get_rule(lines, rule_id):
    rules_ids = [rule_id]
    n_header_lines = 5
    rule = lines[:n_header_lines] # add schema and namespaces
    rule_found = False
    # select all patterns with matching id
    for line in lines[n_header_lines:]:
        if not rule_found:
            # look for beginning of pattern/rule
            match = re.search('pattern id="CSIP(.+?)"', line)
            if match:
                current_id = int(match.group(1))
                if current_id in rules_ids:
                    # select pattern
                    rule.append(line)
                    rule_found = True
                # else ignore pattern
            # else is not the beginning of a pattern
        elif '</pattern>' in line:
            # pattern/rule is over
            rule.append(line)
            rule_found = False
        else:
            rule.append(line)
    rule.append('</schema>\n')
    res = '\n'.join(rule)
    return res


def validate(rules_lines, xml_file, rule_id):
    # Parse rules
    single_rule = io.StringIO(get_rule(rules_lines, rule_id))
    parsed_single_rule = etree.parse(single_rule)
    schematron = isoschematron.Schematron(parsed_single_rule, store_report=True)

    # Parse XML to validate
    parsed_xml_file = etree.parse(xml_file)
    validation_response = schematron.validate(parsed_xml_file)
    report = schematron.validation_report
    return validation_response, report


class TestUtils(unittest.TestCase):
    temp_extract_dir = '/tmp/backend-utils-' + randomword(10)

    @classmethod
    def setUpClass(cls):
        tests_dir = os.path.join(ROOT, 'tests/test_resources/package')
        shutil.copytree(tests_dir, TestUtils.temp_extract_dir)

    @classmethod
    def tearDownClass(cls):
        shutil.rmtree(TestUtils.temp_extract_dir)
        #pass

    def test_recursive_find_files_in_dir(self):

        def check_paths(retrieved_file_list, expected_paths):
            for retrieved_path in retrieved_file_list:
                self.assertTrue(retrieved_path in expected_paths,
                                "Retrieved path %s not in expected list" % retrieved_path)
            for expected_path in expected_paths:
                self.assertTrue(expected_path in retrieved_file_list,
                                "Expected path %s not in retrieved list" % expected_path)

        def check_excluded(retrieved_file_list, excluded_paths):
            for excluded_path in excluded_paths:
                self.assertTrue(excluded_path not in retrieved_file_list,
                                "Excluded path %s appears in retrieved files list" % excluded_path)

        flist = list(rec_find_files(TestUtils.temp_extract_dir, include_files_rgxs=None, exclude_dirsfiles_rgxs=None))

        self.assertEqual(len(flist), 6, "Number of files not as expected")
        check_paths(flist, expected_paths = (
            os.path.join(TestUtils.temp_extract_dir, 'first_level.txt'),
            os.path.join(TestUtils.temp_extract_dir, 'subfolder/second_level.txt'),
            os.path.join(TestUtils.temp_extract_dir, 'subfolder/screen.png'),
            os.path.join(TestUtils.temp_extract_dir, 'subfolder/second_level.csv'),
            os.path.join(TestUtils.temp_extract_dir, 'subfolder/subsubfolder/third_level.txt'),
            os.path.join(TestUtils.temp_extract_dir, 'subfolder/subsubfolder/subsubsubfolder/fourth_level.txt'),
        ))
        with self.assertRaises(AssertionError):
            list(rec_find_files(TestUtils.temp_extract_dir, include_files_rgxs="*.txt", exclude_dirsfiles_rgxs=None))

        flist = list(rec_find_files(TestUtils.temp_extract_dir, include_files_rgxs=[r'.*\.txt$'], exclude_dirsfiles_rgxs=None))
        check_paths(flist, expected_paths = (
            os.path.join(TestUtils.temp_extract_dir, 'first_level.txt'),
            os.path.join(TestUtils.temp_extract_dir, 'subfolder/second_level.txt'),
            os.path.join(TestUtils.temp_extract_dir, 'subfolder/subsubfolder/third_level.txt'),
            os.path.join(TestUtils.temp_extract_dir, 'subfolder/subsubfolder/subsubsubfolder/fourth_level.txt'),
        ))
        check_excluded(flist, excluded_paths=(
            os.path.join(TestUtils.temp_extract_dir, 'subfolder/screen.png'),
        ))
        flist = list(rec_find_files(TestUtils.temp_extract_dir, include_files_rgxs=[r'.*\.txt$', r'.*\.csv$'], exclude_dirsfiles_rgxs=None))
        check_paths(flist, expected_paths = (
            os.path.join(TestUtils.temp_extract_dir, 'first_level.txt'),
            os.path.join(TestUtils.temp_extract_dir, 'subfolder/second_level.txt'),
            os.path.join(TestUtils.temp_extract_dir, 'subfolder/second_level.csv'),
            os.path.join(TestUtils.temp_extract_dir, 'subfolder/subsubfolder/third_level.txt'),
            os.path.join(TestUtils.temp_extract_dir, 'subfolder/subsubfolder/subsubsubfolder/fourth_level.txt'),
        ))
        check_excluded(flist, excluded_paths=(
            os.path.join(TestUtils.temp_extract_dir, 'subfolder/screen.png'),
        ))
        flist = list(rec_find_files(TestUtils.temp_extract_dir, include_files_rgxs=[r'.*\.txt$', r'.*\.csv$'],
                                     exclude_dirsfiles_rgxs=[r'.*/subfolder.*']))
        check_paths(flist, expected_paths=(
            os.path.join(TestUtils.temp_extract_dir, 'first_level.txt'),
        ))

        flist = list(rec_find_files(TestUtils.temp_extract_dir, include_files_rgxs=[r'.*\.txt$'],
                                    exclude_dirsfiles_rgxs=[r'.*first_level.txt$']))
        check_paths(flist, expected_paths=(
            os.path.join(TestUtils.temp_extract_dir, 'subfolder/second_level.txt'),
            os.path.join(TestUtils.temp_extract_dir, 'subfolder/subsubfolder/third_level.txt'),
            os.path.join(TestUtils.temp_extract_dir, 'subfolder/subsubfolder/subsubsubfolder/fourth_level.txt'),
        ))

        flist = list(rec_find_files(TestUtils.temp_extract_dir, include_files_rgxs=[r'.*\.txt$'],
                                    exclude_dirsfiles_rgxs=[r'.*first_level.txt$', r'.*/subsubfolder.*']))
        check_paths(flist, expected_paths=(
            os.path.join(TestUtils.temp_extract_dir, 'subfolder/second_level.txt'),
        ))


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import unittest

from eatb import ROOT
from eatb.csip_validation import CSIPValidation
from eatb.metadata.mets_validation import MetsValidation

from tests.base import Base
from tests.test_utils import validate


class TestMetsValidation(Base):

    def test_csip1(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=1)
        self.assertTrue(validation_result)

    def test_csip2(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=2)
        self.assertTrue(validation_result)

    def test_csip6(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=6)
        self.assertTrue(validation_result)


    def test_csip117(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=117)
        self.assertTrue(validation_result)

    def test_csip7(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=7)
        self.assertTrue(validation_result)

    def test_csip9(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=9)
        self.assertTrue(validation_result)

    def test_csip10(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=10)
        self.assertTrue(validation_result)

    def test_csip11(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=11)
        self.assertTrue(validation_result)

    def test_csip12(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=12)
        self.assertTrue(validation_result)

    def test_csip13(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=13)
        self.assertTrue(validation_result)

    def test_csip14(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=14)
        self.assertTrue(validation_result)

    def test_csip15(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=15)
        self.assertTrue(validation_result)

    def test_csip16(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=16)
        self.assertTrue(validation_result)

    def test_csip18(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=18)
        self.assertTrue(validation_result)

    def test_csip19(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=19)
        self.assertTrue(validation_result)

    def test_csip22(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=22)
        self.assertTrue(validation_result)

    def test_csip23(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=23)
        self.assertTrue(validation_result)

    def test_csip24(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=24)
        self.assertTrue(validation_result)

    def test_csip25(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=25)
        self.assertTrue(validation_result)

    def test_csip26(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=26)
        self.assertTrue(validation_result)

    def test_csip27(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=27)
        self.assertTrue(validation_result)

    def test_csip28(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=28)
        self.assertTrue(validation_result)

    def test_csip29(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=29)
        self.assertTrue(validation_result)

    def test_csip30(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=30)
        self.assertTrue(validation_result)

    def test_csip33(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=33)
        self.assertTrue(validation_result)

    def test_csip36(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=36)
        self.assertTrue(validation_result)

    def test_csip37(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=37)
        self.assertTrue(validation_result)

    def test_csip38(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=38)
        self.assertTrue(validation_result)

    def test_csip39(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=39)
        self.assertTrue(validation_result)

    def test_csip40(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=40)
        self.assertTrue(validation_result)

    def test_csip41(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=41)
        self.assertTrue(validation_result)

    def test_csip42(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=42)
        self.assertTrue(validation_result)

    def test_csip43(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=43)
        self.assertTrue(validation_result)

    def test_csip44(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=44)
        self.assertTrue(validation_result)

    def test_csip46(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=46)
        self.assertTrue(validation_result)

    def test_csip49(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=49)
        self.assertTrue(validation_result)

    def test_csip50(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=50)
        self.assertTrue(validation_result)

    def test_csip51(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=51)
        self.assertTrue(validation_result)

    def test_csip52(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=52)
        self.assertTrue(validation_result)

    def test_csip53(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=53)
        self.assertTrue(validation_result)

    def test_csip54(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=54)
        self.assertTrue(validation_result)

    def test_csip55(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=55)
        self.assertTrue(validation_result)

    def test_csip56(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=56)
        self.assertTrue(validation_result)

    def test_csip57(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=57)
        self.assertTrue(validation_result)

    def test_csip59(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=59)
        self.assertTrue(validation_result)

    def test_csip113(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=113)
        self.assertTrue(validation_result)

    def test_csip114(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=114)
        self.assertTrue(validation_result)

    def test_csip64(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=64)
        self.assertTrue(validation_result)

    def test_csip65(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=65)
        self.assertTrue(validation_result)

    def test_csip66(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=66)
        self.assertTrue(validation_result)

    def test_csip67(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=67)
        self.assertTrue(validation_result)

    def test_csip68(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=68)
        self.assertTrue(validation_result)

    def test_csip69(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=69)
        self.assertTrue(validation_result)

    def test_csip70(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=70)
        self.assertTrue(validation_result)

    def test_csip71(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=71)
        self.assertTrue(validation_result)

    def test_csip72(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=72)
        self.assertTrue(validation_result)

    def test_csip76(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=76)
        self.assertTrue(validation_result)

    def test_csip77(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=77)
        self.assertTrue(validation_result)

    def test_csip78(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=78)
        self.assertTrue(validation_result)

    def test_csip79(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=79)
        self.assertTrue(validation_result)

    def test_csip80(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=80)
        self.assertTrue(validation_result)

    def test_csip81(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=81)
        self.assertTrue(validation_result)

    def test_csip82(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=82)
        self.assertTrue(validation_result, CSIPValidation._get_rule(self.rules_lines, 82))

    def test_csip83(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=83)
        self.assertTrue(validation_result, self.rules_lines)

    def test_csip84(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=84)
        self.assertTrue(validation_result)

    def test_csip85(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=85)
        self.assertTrue(validation_result)

    def test_csip86(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=86)
        self.assertTrue(validation_result)

    def test_csip88(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=88)
        self.assertTrue(validation_result)

    def test_csip89(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=89)
        self.assertTrue(validation_result)

    def test_csip94(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=94)
        self.assertTrue(validation_result)

    def test_csip96(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=96)
        self.assertTrue(validation_result)

    def test_csip116(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=116)
        self.assertTrue(validation_result)

    def test_csip98(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=98)
        self.assertTrue(validation_result)

    def test_csip100(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=100)
        self.assertTrue(validation_result)

    def test_csip118(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=118)
        self.assertTrue(validation_result)

    def test_csip102(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=102)
        self.assertTrue(validation_result)

    def test_csip104(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=104)
        self.assertTrue(validation_result)

    def test_csip119(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=119)
        self.assertTrue(validation_result)

    def test_csip106(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=106)
        self.assertTrue(validation_result)

    def test_csip107(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=107)
        self.assertTrue(validation_result)

    def test_csip108(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=108)
        self.assertTrue(validation_result)

    def test_csip109(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=109)
        self.assertTrue(validation_result)

    def test_csip110(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=110)
        self.assertTrue(validation_result)

    def test_csip111(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=111)
        self.assertTrue(validation_result)

    def test_csip112(self):
        validation_result, _ = validate(self.rules_lines, self.IP_path + '/METS.xml', rule_id=112)
        self.assertTrue(validation_result)

    def test_IP_mets(self):
        mets_file = 'METS.xml'
        mets_schema_file = os.path.join(ROOT, 'tests/test_resources/schemas/mets_1_11.xsd')
        premis_schema_file = os.path.join(ROOT, 'tests/test_resources/schemas/premis-v3-0.xsd')
        mets_validator = MetsValidation(self.IP_path, mets_schema_file, premis_schema_file)
        mets_validator.validate_mets(os.path.join(self.IP_path, mets_file))


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import sys

from eatb import ROOT

sys.path.append(os.path.join(os.path.dirname(__file__), '../'))  # noqa: E402
import unittest

from eatb.utils.datetime import get_file_ctime_iso_date_str, current_timestamp, LengthBasedDateFormat, \
    DT_ISO_FORMAT, ts_date, date_format, current_date


class TestDateTime(unittest.TestCase):

    test_file = os.path.join(ROOT, 'tests/test_resources/test.txt')

    def test_file_ctime_iso_date_str(self):
        """
         Checksum (type MD5) must return SHA256 checksum value
        """

        self.assertEqual(20, len(get_file_ctime_iso_date_str(TestDateTime.test_file)))
        self.assertEqual(19, len(get_file_ctime_iso_date_str(TestDateTime.test_file, "%d.%m.%Y %H:%M:%S")))

    def test_current_timestamp(self):
        self.assertEqual(19, len(current_timestamp()))

    def test_length_based_format(self):
        lbf = LengthBasedDateFormat("2019")
        self.assertEqual("2019-01-01T00:00:00Z", lbf.format(DT_ISO_FORMAT, 4))

    def test_ts_date(self):
        self.assertEqual(19, len(ts_date()))

    def test_date_format(self):
        self.assertEqual(20, len(date_format(current_date())))


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import unittest

from eatb import ROOT
from eatb.checksum import ChecksumFile, ChecksumValidation, ChecksumAlgorithm


class TestChecksum(unittest.TestCase):

    test_dir = os.path.join(ROOT, 'tests/test_resources/')
    test_file = test_dir + 'test.txt'
    csobj = ChecksumFile(test_file)
    csval = ChecksumValidation()

    def test_get_MD5(self):
        """
         Checksum (type MD5) must return SHA256 checksum value
        """
        actual = self.csobj.get(ChecksumAlgorithm.MD5)
        self.assertTrue(actual is not None, "Checksum (type MD5) must not be empty")
        self.assertEqual(actual, "098f6bcd4621d373cade4e832627b4f6")

    def test_get_SHA256(self):
        """
        Checksum (type SHA256) must return SHA256 checksum value
        """
        actual = self.csobj.get(ChecksumAlgorithm.SHA256)
        self.assertTrue(actual is not None, "Checksum (type SHA256) must not be empty")
        self.assertEqual(actual, "9f86d081884c7d659a2feaa0c55ad015a3bf4f1b2b0b822cd15d6c15b0f00a08")

    def test_validate_valid_MD5(self):
        """
        Must validate a correct MD5 checksum
        """
        expected = "098f6bcd4621d373cade4e832627b4f6"
        actual = self.csval.validate_checksum(self.test_file, expected, ChecksumAlgorithm.MD5)
        self.assertTrue(actual, "Must validate a correct MD5 checksum")

    def test_notvalidate_invalid_MD5(self):
        """
        Must validate a correct MD5 checksum
        """
        expected = "incorrect_md5"
        actual = self.csval.validate_checksum(self.test_file, expected, ChecksumAlgorithm.MD5)
        self.assertFalse(actual, "Must not validate an incorrect MD5 checksum")

    def test_validate_valid_SHA256(self):
        """
        Must validate a correct SHA256 checksum
        """
        expected = "9f86d081884c7d659a2feaa0c55ad015a3bf4f1b2b0b822cd15d6c15b0f00a08"
        actual = self.csval.validate_checksum(self.test_file, expected, ChecksumAlgorithm.SHA256)
        self.assertTrue(actual, "Must validate a correct SHA256 checksum")

    def test_notvalidate_invalid_SHA256(self):
        """
        Must validate a correct SHA256 checksum
        """
        expected = "incorrect_sha256"
        actual = self.csval.validate_checksum(self.test_file, expected, ChecksumAlgorithm.SHA256)
        self.assertFalse(actual, "Must not validate an incorrect SHA256 checksum")


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import unittest

try:
    from fido.fido import Fido
    fido_disabled = False
except ImportError:
    fido_disabled = True

from eatb import ROOT
from eatb.file_format import FormatIdentification


class TestFormatIdentification(unittest.TestCase):

    def testFido(self):
        if not fido_disabled:
            ffid = FormatIdentification()
            puid = ffid.identify_file(os.path.join(ROOT, "tests/test_resources/schemas/xlink.xsd"))
            self.assertTrue("x-fmt/280", puid)
            self.assertEqual("application/xml", ffid.get_mime_for_puid(puid))
        else:
            print("Note: Fido module is not available!")


if __name__ == '__main__':
    unittest.main()



import os
import re

import lxml

from eatb.utils.fileutils import get_sub_path_from_relative_path


class ParsedEad():
    """
    Parsed EAD object
    """
    ead_tree = None
    ead_file_path = None

    def __init__(self, rdir, ead_file_path):
        """

        :param rdir: Path to root directory (relative links in the EAD are evaluated according to this path)
        :param ead_file_path: Path to the EAD metadata file
        """
        self.ns = {'ead': 'http://ead3.archivists.org/schema/',
                   'xlink': 'http://www.w3.org/1999/xlink',
                   'xsi': 'http://www.w3.org/2001/XMLSchema-instance'}

        self.root_dir = rdir
        self.ead_tree = None
        self.ead_file_path = ead_file_path
        self._load_ead(ead_file_path)

    def _load_ead(self, ead_file_path):
        """
        Load ead Element tree object

        @type       ead_file_path: ElementTree
        @param      ead_file_path: Parsed EAD
        """
        if not os.path.exists(ead_file_path):
            raise ValueError('The XML file is not available at the given location: %s' % ead_file_path)
        self.ead_tree = lxml.etree.parse(ead_file_path)

    def get_root(self):
        if self.ead_tree is None:
            raise ValueError("Attribute 'ead_tree' of type ElementTree must be initialized")
        return self.ead_tree.getroot()

    def get_dao_elements(self):
        return self.ead_tree.getroot().xpath('//ead:dao', namespaces=self.ns)

    def _first_md_val_ancpath(self, current_elm, md_tag, text_accessor=None, is_attr_text_accessor=False):
        def get_text_val(node):
            if not text_accessor:
                return node.text
            tc_elms = node.findall(text_accessor, namespaces=self.ns)
            return None if len(tc_elms) != 1 else tc_elms[0].text
        parent_elms = current_elm.findall("..")
        if parent_elms is not None and len(parent_elms) == 1:
            parent = parent_elms[0]
            for child in parent:
                if re.match(md_tag, child.xpath('local-name()'), re.IGNORECASE):
                    if is_attr_text_accessor:
                        return child.get(text_accessor)
                    return get_text_val(child)
                for c in child:
                    if re.match(md_tag, c.xpath('local-name()'), re.IGNORECASE):
                        return get_text_val(c)
            if re.match(md_tag, parent.tag, re.IGNORECASE):
                return parent.get(text_accessor)
            return self._first_md_val_ancpath(parent, md_tag, text_accessor,
                                              is_attr_text_accessor)
        return None

    def dao_path_mdval_tuples(self, md_tag, text_val_sub_path=None, is_attr_text_accessor=False):
        dao_elements = self.get_dao_elements()
        result = []
        for dao_elm in dao_elements:
            path = get_sub_path_from_relative_path(self.root_dir, self.ead_file_path, dao_elm.attrib['href'])
            mdval = self._first_md_val_ancpath(dao_elm, md_tag, text_val_sub_path, is_attr_text_accessor)
            if mdval:
                result.append({"path": path, "mdvalue": mdval})
        return result



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import shutil
import unittest

from pairtree import ObjectNotFoundException

from eatb import ROOT
from eatb.pairtree_storage import PairtreeStorage
from eatb.utils import randomutils

source_dir = os.path.join(ROOT, 'tests/test_resources/storage-test/')
package_file = "xyz.tar"
package_file_path = os.path.join(source_dir, package_file)
repository_storage_dir = '/tmp/temp-' + randomutils.randomword(10)
test_repo = os.path.join(ROOT, 'tests/test_resources/test-repo/')


class TestPairtreeStorage(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        if not os.path.exists(repository_storage_dir):
            os.makedirs(repository_storage_dir)
        shutil.copy(os.path.join(test_repo, "pairtree_version0_1"), repository_storage_dir)

    @classmethod
    def tearDownClass(cls):
        shutil.rmtree(repository_storage_dir)

    def test_identifier_object_exists(self):
        pts = PairtreeStorage(test_repo)
        existing_identifier = "bar"
        non_existing_identifier = "foo"
        existing_identifier_exists = pts.identifier_object_exists(existing_identifier)
        non_existing_identifier_exists = pts.identifier_object_exists(non_existing_identifier)
        self.assertEqual(existing_identifier_exists, True)
        self.assertEqual(non_existing_identifier_exists, False)

    def test_version_exists(self):
        pts = PairtreeStorage(test_repo)
        identifier = "bar"
        self.assertEqual(pts.identifier_version_object_exists(identifier, 3),False)
        self.assertEqual(pts.identifier_version_object_exists(identifier, 2),True)

    def test_next_version(self):
        pts = PairtreeStorage(test_repo)
        identifier = "bar"
        self.assertEqual("v00003", pts._next_version(identifier))

    def test_curr_version(self):
        pts = PairtreeStorage(test_repo)
        identifier = "bar"
        self.assertEqual("v00002", pts.curr_version(identifier))

    def test_store(self):
        pts = PairtreeStorage(repository_storage_dir)
        pts.store("xyz", package_file_path)
        self.assertEqual(0, pts.curr_version_num("xyz"))
        pts.store("xyz", package_file_path)
        self.assertEqual(1, pts.curr_version_num("xyz"))

    def test_get_object_path(self):
        pts = PairtreeStorage(test_repo)
        expected = os.path.join(test_repo, "pairtree_root/ba/r/data/v00002/bar.tar")
        actual = pts.get_object_path("bar")
        self.assertEqual(expected, actual)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import configparser
import os

from eatb import ROOT

config = configparser.ConfigParser()
config['DEFAULT'] = {}
config.sections()

config.read(os.path.join(ROOT, 'eatb/settings.cfg'))

application_name = config.get('global', 'application_name')
application_version = config.get('global', 'application_version')

fido_enabled = bool(eval(config.get('format', 'fido_enabled')))

format_files_cfg = config.get('format', 'format_files')

validation_profile = config.get('validation', 'validation_profile')




#!/usr/bin/env python
# -*- coding: utf-8 -*-
import fnmatch
import os

import eatb.metadata

from eatb.metadata.parsed_mets import ParsedMets
from eatb.metadata.mets_validation import XLINK_NS, METS_NS
from eatb.utils.fileutils import rec_find_files


folders_with_USE = ['documentation', 'schemas', 'representations', "data"]

logger = eatb.metadata.logger


def get_package_mets_files_from_basedir(base_directory):
        """
        Get list of information package METS file paths based on a directory which contains information packages in subdirectories.

        @type       base_directory: str
        @param      base_directory: Directory which contains information packages in subdirectories.
        @rtype:     list
        @return:    String list of information package METS file paths
        """
        return list(rec_find_files(base_directory, include_files_rgxs=[r'.*METS.xml$'],
                                    exclude_dirsfiles_rgxs=None))


def get_mets_obj_id(mets_file_path):
        """
        Get identifier from mets file

        @type       mets_file_path: str
        @param      mets_file_path: METS file path
        @rtype:     str
        @return:    Object identifier
        """
        package_path, file_name = os.path.split(mets_file_path)
        pm = ParsedMets(package_path)
        pm.load_mets(mets_file_path)
        return str(pm.get_obj_id())


def get_mets_objids_from_basedir(base_directory):
        """
        Get list of OBJID strings from METS files based on a directory which contains information packages in subdirectories.

        @type       base_directory: str
        @param      base_directory: Directory which contains information packages in subdirectories.
        @rtype:     list
        @return:    Object identifier
        """
        mets_file_paths = get_package_mets_files_from_basedir(base_directory)
        mets_obj_ids = []
        for mets_file_path in mets_file_paths:
            mets_obj_ids.append(get_mets_obj_id(mets_file_path))
        return mets_obj_ids


def get_folder_with_USE(path):
    try:
        dirs = path.split('/')
        for dir in dirs[::-1]:
            if dir in folders_with_USE:
                return dir
    except Exception as e:
        logger.debug('Parent directory not used')
        logger.debug(e)
    return 'other'





#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os

import unittest

from eatb import ROOT
from eatb.utils.fileutils import remove_protocol, strip_prefixes, get_sub_path_from_relative_path


class TestPathFunctions(unittest.TestCase):

    def test_remove_protocol(self):
        self.assertEqual('/test', remove_protocol("file:///test"))

    def test_strip_prefixes_from_path_singlefile(self):
        full_path = '/to/be/stripped/someotherpart/file.txt'
        path_part1 = '/to/be/stripped'
        path_part2 = 'someotherpart'
        self.assertEqual('file.txt', strip_prefixes(full_path, path_part1, path_part2))

    def test_strip_prefixes_from_path_file_in_subdir(self):
        full_path = '/to/be/stripped/someotherpart/sub/dir/file.txt'
        path_part1 = '/to/be/stripped'
        path_part2 = 'someotherpart'
        self.assertEqual('sub/dir/file.txt', strip_prefixes(full_path, path_part1, path_part2))

    def test_strip_single_prefix_from_path(self):
        full_path = '/to/be/stripped/someotherpart/file.txt'
        path_part = '/to/be/stripped'
        self.assertEqual('someotherpart/file.txt', strip_prefixes(full_path, path_part))

    def test_package_path_from_relative_path(self):
        test_ip = "eark-ip"
        test_source_directory = os.path.join(ROOT + '/tests/test_resources/')

        test_ip_root = os.path.join(test_source_directory, test_ip)

        containing_file_path = os.path.join(test_ip_root, 'metadata/descriptive/EAD.xml')

        relative_path = '../../submission/metadata/descriptive/EAD.xml'
        self.assertEqual("submission/metadata/descriptive/EAD.xml",
                         get_sub_path_from_relative_path(test_ip_root, containing_file_path, relative_path))


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os

import shutil
import unittest

from eatb import ROOT
from eatb.utils import randomutils
from eatb.packaging import PackagedContainer


class TestExtract(unittest.TestCase):
    test_dir = os.path.join(ROOT, 'tests/test_resources/')
    temp_extract_dir = '/tmp/temp-' + randomutils.randomword(10)

    @classmethod
    def setUpClass(cls):
        if not os.path.exists(TestExtract.temp_extract_dir):
            os.makedirs(TestExtract.temp_extract_dir)

    @classmethod
    def tearDownClass(cls):
        shutil.rmtree(TestExtract.temp_extract_dir)
        pass

    @staticmethod
    def reset_test_dir():
        shutil.rmtree(TestExtract.temp_extract_dir)
        os.makedirs(TestExtract.temp_extract_dir)

    def assert_extract(self, package_file):
        package_file_path = os.path.join(TestExtract.test_dir, package_file)
        packaged_container = PackagedContainer.factory(package_file_path)
        packaged_container.extract(TestExtract.temp_extract_dir)
        # must be 8 extracted files
        item_counts = [(len(r), len(d), len(files)) for r, d, files in os.walk(TestExtract.temp_extract_dir)]
        sum_files = sum(ic[2] for ic in item_counts)
        self.assertEqual(sum_files, 2, "Number of extracted files not as expected")
        sum_directories = sum(ic[1] for ic in item_counts)
        self.assertEqual(sum_directories, 2, "Number of extracted directories not as expected")
        files_to_check = (
            os.path.join(TestExtract.temp_extract_dir, 'package/first_level.txt'),
            os.path.join(TestExtract.temp_extract_dir, 'package/subfolder/second_level.txt'),
        )
        for file in files_to_check:
            self.assertTrue(os.path.isfile(file), "File %s not found in extracted directory" + file)
        TestExtract.reset_test_dir()

    def test_extract_zip(self):
        self.assert_extract("package.zip")

    def test_extract_tar(self):
        self.assert_extract("package.tar.gz")


if __name__ == '__main__':
    unittest.main()



import os
from xml.dom import minidom
from xml.dom.minidom import parseString
from xml.etree import ElementTree

import lxml.etree as ET

from eatb.utils.fileutils import read_file_content


def pretty_xml_string(xml_string):
    xml = parseString(xml_string)
    return xml.toprettyxml()

def prettify(elem):
    """Return a pretty-printed XML string for the Element.
    """
    rough_string = ElementTree.tostring(elem, 'utf-8')
    reparsed = minidom.parseString(rough_string)
    return reparsed.toprettyxml(indent="  ")


def rewrite_pretty_xml(xml_file_path):
    parser = ET.XMLParser(resolve_entities=False, remove_blank_text=True, strip_cdata=False)
    parsed_file = ET.parse(xml_file_path, parser)
    xml_file_root = parsed_file.getroot()
    xml_content = ET.tostring(xml_file_root, encoding='UTF-8', pretty_print=True, xml_declaration=True)
    with open(xml_file_path, 'w') as output_file:
        output_file.write(xml_content)
        output_file.close()


def get_xml_schemalocations(xml_file):
    XSI = "http://www.w3.org/2001/XMLSchema-instance"
    xml_dir, tail = os.path.split(xml_file)
    xml_content = read_file_content(xml_file)
    tree = ET.XML(xml_content)
    schema_locs = []
    #Find instances of xsi:schemaLocation
    schema_locations = set(tree.xpath("//*/@xsi:schemaLocation", namespaces={'xsi': XSI}))
    for schema_location in schema_locations:
        namespaces_locations = schema_location.strip().split()
        # Import all fnamspace/schema location pairs
        for namespace, location in zip(*[iter(namespaces_locations)] * 2):
            loc = os.path.abspath(os.path.join(xml_dir, location))
            if not os.path.exists(loc):
                print("ERROR: XML-Schema file not found: %s" % loc)
                raise XMLSchemaNotFound("XML-Schema file not found: %s" % loc)
            schema_locs.append(loc)
    return schema_locs


def change_dcat_element_values(md_file, substitutions):

    xml_content = read_file_content(md_file)
    tree = ET.XML(xml_content)

    for xpath_expr, value in substitutions.iteritems():
        schema_locations = set(tree.xpath(xpath_expr, namespaces={'dct': 'http://purl.org/dc/terms/', 'dcat': 'http://www.w3.org/ns/dcat#'}))
        for schema_location in schema_locations:
            print(schema_location)
            schema_location.text = value
            sl = schema_location
    xml_content = ET.tostring(tree, encoding='UTF-8', pretty_print=True, xml_declaration=True)
    with open(md_file, 'w') as output_file:
        output_file.write(xml_content)
        output_file.close()


def get_dcat_element_values(md_file, substitutions):

    xml_content = read_file_content(md_file)
    tree = ET.XML(xml_content.encode('utf-8'))

    for xpath_expr, value in substitutions.items():
        schema_locations = set(tree.xpath(xpath_expr, namespaces={'dct': 'http://purl.org/dc/terms/', 'dcat': 'http://www.w3.org/ns/dcat#'}))
        for schema_location in schema_locations:
            print(schema_location)
            schema_location.text = value
            sl = schema_location
    xml_content = ET.tostring(tree, encoding='UTF-8', pretty_print=True, xml_declaration=True)
    return xml_content


class XMLSchemaNotFound(Exception):

    def __init__(self, value):
        self.parameter = value

    def __str__(self):
        return repr(self.parameter)


class XmlDictObject(dict):

    def __init__(self, initdict=None):
        if initdict is None:
            initdict = {}
        dict.__init__(self, initdict)

    def __getattr__(self, item):
        return self.__getitem__(item)

    def __setattr__(self, item, value):
        self.__setitem__(item, value)

    def __str__(self):
        if self.has_key('_text'):
            return self.__getitem__('_text')
        else:
            return ''

    @staticmethod
    def Wrap(x):

        if isinstance(x, dict):
            return XmlDictObject((k, XmlDictObject.Wrap(v)) for (k, v) in x.items())
        elif isinstance(x, list):
            return [XmlDictObject.Wrap(v) for v in x]
        else:
            return x

    @staticmethod
    def _UnWrap(x):
        if isinstance(x, dict):
            return dict((k, XmlDictObject._UnWrap(v)) for (k, v) in x.items())
        elif isinstance(x, list):
            return [XmlDictObject._UnWrap(v) for v in x]
        else:
            return x

    def UnWrap(self):
        return XmlDictObject._UnWrap(self)


def _ConvertDictToXmlRecurse(parent, dictitem):
    assert type(dictitem) is not type([])

    if isinstance(dictitem, dict):
        for (tag, child) in dictitem.items():
            if str(tag) == '_text':
                parent.text = str(child)
            elif type(child) is type([]):
                # iterate through the array and convert
                for listchild in child:
                    elem = ElementTree.Element(tag)
                    parent.append(elem)
                    _ConvertDictToXmlRecurse(elem, listchild)
            else:
                elem = ElementTree.Element(tag)
                parent.append(elem)
                _ConvertDictToXmlRecurse(elem, child)
    else:
        parent.text = str(dictitem)


def ConvertDictToXml(xmldict):
    roottag = xmldict.keys()[0]
    root = ElementTree.Element(roottag)
    _ConvertDictToXmlRecurse(root, xmldict[roottag])
    return root


def _ConvertXmlToDictRecurse(node, dictclass):
    nodedict = dictclass()

    if len(node.items()) > 0:
        nodedict.update(dict(node.items()))

    for child in node:
        newitem = _ConvertXmlToDictRecurse(child, dictclass)
        if nodedict.has_key(child.tag):
            if type(nodedict[child.tag]) is type([]):
                nodedict[child.tag].append(newitem)
            else:
                nodedict[child.tag] = [nodedict[child.tag], newitem]
        else:
            nodedict[child.tag] = newitem

    if node.text is None:
        text = ''
    else:
        text = node.text.strip()

    if len(nodedict) > 0:
        if len(text) > 0:
            nodedict['_text'] = text
    else:
        nodedict = text

    return nodedict


def ConvertXmlToDict(root, dictclass=XmlDictObject):
    if type(root) == type(''):
        root = ElementTree.parse(root).getroot()
    elif not isinstance(root, ElementTree.Element):
        raise TypeError('Expected ElementTree.Element or file path string')

    return dictclass({root.tag: _ConvertXmlToDictRecurse(root, dictclass)})


def MetaIdentification(unknown_xml):
    parsed_unknown = ET.parse(unknown_xml)
    unknown_tag =  parsed_unknown.getroot().tag
    tag = unknown_tag.split('}')
    if len(tag) == 2:
        return tag[1]
    if len(tag) == 1:
        return tag[0]



#!/usr/bin/env python
# coding=UTF-8
import sys

class CC:
    def __init__(self):
        pass
    HEADER = '\033[94m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'


def print_headline(headline):
    print("%s====================================================================%s" % (CC.HEADER, CC.ENDC))
    print("%s%s%s" % (CC.HEADER, headline, CC.ENDC))
    print("%s====================================================================%s" % (CC.HEADER, CC.ENDC))


def success(msg):
    print("%s%s%s" % (CC.OKGREEN, msg, CC.ENDC))


def failure(msg):
    print("%s%s%s" % (CC.FAIL, msg, CC.ENDC))
    sys.exit(1)


def warning(msg):
    print("%s%s%s" % (CC.WARNING, msg, CC.ENDC))



import fnmatch
import os

from lxml import etree

from eatb import ROOT
from eatb.checksum import ChecksumValidation
from eatb.utils.XmlHelper import q
from eatb.utils.fileutils import remove_protocol

XLINK_NS = "http://www.w3.org/1999/xlink"
METS_NS = 'http://www.loc.gov/METS/'


class MetsValidation(object):
    '''
    Validation of the Mets file. This includes also the validation of Premis files that are linked in the amdSec!
    '''

    def __init__(self, root, mets_schema_file=os.path.join(ROOT, "eatb/resources/schemas/mets.xsd"),
                 premis_schema_file=os.path.join(ROOT, "eatb/resources/schemas/premis-v3-0.xsd")):
        self.validation_errors = []
        self.total_files = 0
        self.schema_mets = etree.XMLSchema(file=mets_schema_file)
        self.schema_premis = etree.XMLSchema(file=premis_schema_file)
        self.rootpath = root
        self.subsequent_mets = []

    def validate_mets(self, mets):
        '''
        Validates a Mets file. The Mets file is parsed with etree.iterparse(), which allows event-driven parsing of
        large files. On certain events/conditions actions are taken, like file validation or adding Mets files found
        inside representations to a list so that they will be evaluated later on.

        @param mets:    Path leading to a Mets file that will be evaluated.
        @return:        Boolean validation result.
        '''
        self.rootpath = mets.rsplit('/', 1)[0]

        try:
            parsed_mets = etree.iterparse(mets, events=('start', 'end'), schema=self.schema_mets)
            for event, element in parsed_mets:
                # Define what to do with specific tags.
                if event == 'end' and element.tag == q(METS_NS, 'file'):
                    # files
                    self.total_files += 1
                    self.validate_file(element)
                    element.clear()
                    while element.getprevious() is not None:
                        del element.getparent()[0]
                elif event == 'end' and element.tag == q(METS_NS, 'div') and element.attrib['LABEL'].startswith('representations/'):
                    if fnmatch.fnmatch(element.attrib['LABEL'].rsplit('/', 1)[1], '*_mig-*'):
                        # representation mets files
                        rep = element.attrib['LABEL'].rsplit('/', 1)[1]
                        for child in element.getchildren():
                            if child.tag == q(METS_NS, 'mptr'):
                                metspath = child.attrib[q(XLINK_NS, 'href')]
                                sub_mets = rep, metspath
                                self.subsequent_mets.append(sub_mets)
                        element.clear()
                        while element.getprevious() is not None:
                            del element.getparent()[0]
                elif event == 'end' and element.tag == q(METS_NS, 'dmdSec'):
                    # dmdSec
                    pass
                elif event == 'end' and element.tag == q(METS_NS, 'amdSec'):
                    # pass
                    if len(element.getchildren()) > 0:
                        for child in element.getchildren():
                            # child are: digiprovMD
                            if len(child.getchildren()) > 0:
                                for sub_child in child.getchildren():
                                    # sub_child are: mdRef
                                    if sub_child.tag == etree.Comment or sub_child.tag == etree.PI:  # filter out comments (they also count as children)
                                        pass
                                    elif sub_child.attrib['MDTYPE'] == 'PREMIS':
                                        if not sub_child.attrib[q(XLINK_NS, 'href')].startswith('/'):
                                            rel_path = sub_child.attrib[q(XLINK_NS, 'href')]
                                            premis = os.path.join(self.rootpath, rel_path[2:])
                                            try:
                                                parsed_premis = etree.iterparse(premis, events=('start',), schema=self.schema_premis)
                                                for event, el in parsed_premis:
                                                    # TODO: Validate against AIP PREMIS Specification
                                                    pass
                                                print('Successfully validated Premis file: %s' % premis)

                                            except etree.XMLSyntaxError as e:
                                                print('VALIDATION ERROR: The Premis file %s yielded errors:' % premis)
                                                print(e)
                                                self.validation_errors.append(e)
                                        else:
                                            pass
                                    else:
                                        pass
        except etree.XMLSyntaxError as e:
            self.validation_errors.append(e)
        except BaseException as e:
            self.validation_errors.append(e)

        if self.total_files != 0:
            self.validation_errors.append('File count yielded %d instead of 0.' % self.total_files)

        if self.validation_errors and len(self.validation_errors) > 0:
            print('Error log for METS file: ', mets)
            for error in self.validation_errors:
                print(error)

        return True if len(self.validation_errors) == 0 else False


    def validate_file(self, file):
        '''
        Validates every file found inside a Mets, so far: size, checksum, fixity. If a file exists, the counter for
        self.total_files is diminished.

        @param file:    XML Element of a file that will be validated.
        @return:
        '''
        err = []
        log = []

        # get information about the file
        for child in file.getchildren():
            if child.tag == etree.Comment or child.tag == etree.PI:
                # skip if it's an XML comment
                pass
            elif child.tag == q(METS_NS, 'FLocat'):
                attr_path = child.attrib[q(XLINK_NS, 'href')]
        attr_size = file.attrib['SIZE']
        attr_checksum = file.attrib['CHECKSUM'].lower() # just in case someone creates a checksum with uppercase letters
        attr_checksumtype = file.attrib['CHECKSUMTYPE']
        # mimetpye = file.attrib['MIMETYPE']

        # check if file exists, if yes validate it
        fitem = remove_protocol(attr_path)
        file_path = os.path.join(self.rootpath, fitem).replace('\\', '/')

        if not os.path.exists(file_path):
            err.append("Unable to find file referenced in METS: %s" % file_path)
        else:
            self.total_files -= 1
            # check if file size is valid
            # TODO: is this even needed?
            file_size = os.path.getsize(file_path)
            if not int(file_size) == int(attr_size):
                err.append("Actual file size %s does not equal file size attribute value %s, file: %s" % (file_size, attr_size, file_path))
                # workaround for conduit.log in AIP metadata/ folder on IP root level
                if file_path[-20:] == 'metadata/conduit.log':
                    err.pop()
                    log.append('Forced validation result \'True\' for file: %s' % (file_path))

            # validate checksum
            checksum_validation = ChecksumValidation()
            checksum_result = checksum_validation.validate_checksum(file_path, attr_checksum, attr_checksumtype)

            # workaround for conduit.log in AIP metadata/ folder on IP root level
            if file_path[-20:] == 'metadata/conduit.log':
                checksum_result = True

            if not checksum_result == True:
                err.append('Checksum validation failed for: %s' % file_path)

        for error in err:
            print('File validation error: ' + error)
            self.validation_errors.append(error)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import shutil
import subprocess
import unittest

from eatb import ROOT
from eatb.cli import CliCommand, CliCommands
from eatb.utils import randomutils
from eatb.utils.encrypt import gpg_encrypt_file_passphrase
from eatb.utils.fileutils import read_file_content


class EncryptionTest(unittest.TestCase):

    test_file = 'test.txt'
    test_dir = os.path.join(ROOT, 'tests/test_resources')
    test_file_path = os.path.join(test_dir, test_file)
    tmp_dir = '/tmp/temp-' + randomutils.randomword(10)
    tmp_file_path = os.path.join(tmp_dir, test_file)

    def setUp(self):
        os.makedirs(EncryptionTest.tmp_dir, exist_ok=True)
        shutil.copyfile(EncryptionTest.test_file_path, EncryptionTest.tmp_file_path)

    def tearDown(self):
        shutil.rmtree(EncryptionTest.tmp_dir)

    def test_gpg_encrypt_file_passphrase(self):
        """
        Test encrypt/decrypt with passphrase
        """
        cmd_encrypt = gpg_encrypt_file_passphrase(EncryptionTest.test_file, "12345")
        print(cmd_encrypt)
        self.assertEqual(7, len(cmd_encrypt))
        expected_cmd_list = ['gpg', '--yes', '--batch', '--passphrase', '12345', '-c', EncryptionTest.test_file]
        self.assertTrue(cmd_encrypt == expected_cmd_list)
        out = subprocess.call(cmd_encrypt, cwd=EncryptionTest.tmp_dir)
        self.assertEqual(0, out)
        cli_commands = CliCommands()
        cli_command = CliCommand("gpg_passphrase_decrypt_file", cli_commands.get_command_template("gpg_passphrase_decrypt_file"))
        cmd_decrypt = cli_command.get_command({'encrypted_file': "%s.gpg" % EncryptionTest.test_file,
                                               'decrypted_file': "test_decrypted.txt", 'passphrase': "12345"})
        out = subprocess.call(cmd_decrypt, cwd=EncryptionTest.tmp_dir)
        self.assertEqual(0, out)
        self.assertEqual("test", read_file_content(os.path.join(EncryptionTest.tmp_dir,"test_decrypted.txt")))


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from lxml import objectify

XSI_NS = 'http://www.w3.org/2001/XMLSchema-instance'

def q(ns, v):
    return '{{{}}}{}'.format(ns, v)

def sequence_insert(node, child, successor_sections):
    insert_function = node.append
    for section in successor_sections:
        path = objectify.ObjectPath(node.tag + '.' + section)
        if path.hasattr(node):
            insert_function = node[section].addprevious
            break
    insert_function(child)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import hashlib
import os

from eatb import logger
from eatb.utils.fileutils import fsize


class ChecksumAlgorithm:
    """
    Checksum algorithm
    """
    MD5, SHA256, SHA512, NONE = range(4)

    @staticmethod
    def get(alg):
        """
        Get algorithm by string
        :param alg: Algorithm string
        :return: Checksum algorithm
        """
        if alg.lower() == "md5":
            return ChecksumAlgorithm.MD5
        if alg.lower() == "sha256" or alg.lower() == "sha-256":
            return ChecksumAlgorithm.SHA256
        if alg.lower() == "sha256" or alg.lower() == "sha-256":
            return ChecksumAlgorithm.SHA512
        return ChecksumAlgorithm.NONE

    @staticmethod
    def str(alg):
        if alg is ChecksumAlgorithm.MD5:
            return "MD5"
        if alg is ChecksumAlgorithm.SHA256:
            return "SHA256"
        if alg is ChecksumAlgorithm.SHA256:
            return "SHA512"
        return "NONE"


class ChecksumFile():
    """
    Checksum validation
    """

    blocksize = 65536

    def __init__(self, file_path):
        """
        Constructor takes file path as argument; checksums of different types can then be calculated for this file.
        :param file_path: Path to file
        """
        self.file_path = file_path

    def get(self, checksum_algorithm):
        """
        This function calculates the checksum using the algorithm specified by the argument 'checksum_algorithm'.
        :param checksum_algorithm: Algorithm used to create the checksum
        :return: Checksum string
        """
        hashval = None
        if checksum_algorithm in [ChecksumAlgorithm.SHA256, 'SHA-256']:
            hashval = hashlib.sha256()
        if checksum_algorithm in [ChecksumAlgorithm.SHA512, 'SHA-512']:
            hashval = hashlib.sha512()
        elif checksum_algorithm in [ChecksumAlgorithm.MD5, 'MD5']:
            hashval = hashlib.md5()

        with open(self.file_path, 'rb') as f:
            while True:
                buf = f.read(self.blocksize)
                if not buf:
                    break
                hashval.update(buf)
        return hashval.hexdigest()


class ChecksumValidation():
    """
    Checksum validation
    """

    def validate_checksum(self, file_path, checksum_expected, checksum_algorithm):
        """

        :param file_path: Path to file
        :param checksum_expected: Expected checksum
        :param checksum_algorithm: Algorithm used to create the checksum
        :return: Validity of checksum (bool)
        """
        checksum_file = ChecksumFile(file_path)
        calculated_checksum = checksum_file.get(checksum_algorithm)
        return calculated_checksum == checksum_expected


def get_hash(file, checksum_algorithm):
    """
    Get MD5/SHA256/SHA512 hash
    :param file: Path to file
    :return: MD5/SHA256/SHA512 hash
    """
    blocksize = 65536
    hashval = None
    if checksum_algorithm in [ChecksumAlgorithm.SHA256, 'SHA-256']:
        hashval = hashlib.sha256()
    if checksum_algorithm in [ChecksumAlgorithm.SHA512, 'SHA-512']:
        hashval = hashlib.sha512()
    elif checksum_algorithm in [ChecksumAlgorithm.MD5, 'MD5']:
        hashval = hashlib.md5()
    with open(file, 'rb') as file:
        while True:
            buf = file.read(blocksize)
            if not buf:
                break
            hashval.update(buf)
    return hashval.hexdigest()


def get_hash_values(file):
    """
    Get MD5/SHA256/SHA512 hash
    :param file: Path to file
    :return: MD5/SHA256/SHA512 hash
    """
    blocksize = 65536
    hashval_sha256 = hashlib.sha256()
    hashval_md5 = hashlib.md5()
    hashval_sha512 = hashlib.sha512()
    with open(file, 'rb') as file:
        while True:
            buf = file.read(blocksize)
            if not buf:
                break
            hashval_md5.update(buf)
            hashval_sha256.update(buf)
            hashval_sha512.update(buf)
    return hashval_md5.hexdigest(), hashval_sha256.hexdigest(), hashval_sha512.hexdigest()

def get_md5_hash(file):
    """
    Get MD5 hash
    :param file: Path to file
    :return: MD5 hash
    """
    return get_hash(file, ChecksumAlgorithm.MD5)


def get_sha256_hash(file):
    """
    Get SHA256 hash
    :param file: Path to file
    :return: SHA256 hash
    """
    return get_hash(file, ChecksumAlgorithm.SHA256)


def get_sha512_hash(file):
    """
    Get SHA512 hash
    :param file: Path to file
    :return: SHA512 hash
    """
    return get_hash(file, ChecksumAlgorithm.SHA512)


def checksum(file_path, wd=None, alg=ChecksumAlgorithm.SHA256):
    """
    Get checksum
    :param file_path: file path
    :param wd: working directory
    :param alg: algorithm
    :return:
    """
    fp = file_path
    path = fp if wd is None else os.path.join(wd, fp)
    return ChecksumFile(path).get(alg)


def check_transfer(source, target):
    """
    Check successful transfer by comparing checksum values and file sizes of source and target
    :param file_path: file path
    :param wd: working directory
    :param alg: algorithm
    :except Different file sizes or checksum values
    :return: None
    """
    source_size = fsize(source)
    target_size = fsize(target)
    if not source_size == target_size:
        raise ValueError("Size of source file (%d bytes) and size of target file (%d bytes) are not equal" %
                         (source_size, target_size))
    if not ChecksumFile(source).get(ChecksumAlgorithm.SHA256) == ChecksumFile(target).get(ChecksumAlgorithm.SHA256):
        raise ValueError("Checksums of source %s and target %s are not equal" % (source, target))


def files_identical(file1, file2):
    if not (os.path.exists(file1) and os.path.exists(file2)):
        logger.error("Files must both exist to verify if they are identical: %s, %s" % (file1, file2))
        return False
    checksum_source_file = ChecksumFile(file1).get('SHA-256')
    checksum_target_file = ChecksumFile(file2).get('SHA-256')
    logger.debug("f1: %s, f2: %s" % (file1, file2))
    logger.debug("cs1: %s, cs2: %s" % (checksum_source_file, checksum_target_file))
    return checksum_source_file == checksum_target_file



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import shutil
import unittest

from lxml import etree

from eatb import ROOT
from eatb.metadata.premis_generator import PremisGenerator
from eatb.utils import randomutils


class TestPremisCreation(unittest.TestCase):
    test_ip = "eark-ip"
    test_ip_mig = "eark-ip-mig"
    test_source_directory = os.path.join(ROOT + '/tests/test_resources/')
    temp_working_dir = os.path.join(ROOT, "tmp/tmp-%s" % randomutils.randomword(10))
    tmp_ip_dir = os.path.join(temp_working_dir, test_ip)
    tmp_ip_dir_mig = os.path.join(temp_working_dir, test_ip_mig)

    test_ip_dir = os.path.join(test_source_directory, test_ip)
    test_ip_dir_mig = os.path.join(test_source_directory, test_ip_mig)

    premis_file_path = os.path.join(temp_working_dir, test_ip, './metadata/preservation/premis.xml')

    premis_schema_file = os.path.join(ROOT, 'tests/test_resources/schemas/premis-v3-0.xsd')

    @classmethod
    def setUpClass(cls):
        shutil.copytree(TestPremisCreation.test_ip_dir, TestPremisCreation.tmp_ip_dir)
        shutil.copytree(TestPremisCreation.test_ip_dir_mig, TestPremisCreation.tmp_ip_dir_mig)

    @classmethod
    def tearDownClass(cls):
        shutil.rmtree(TestPremisCreation.temp_working_dir)
        pass

    def validate_xml(self, file, premis_schema_file):
        validation_errors = []
        xmlschema = etree.XMLSchema(etree.parse(premis_schema_file))
        try:
            xmlschema.assertValid(etree.parse(file))
            print('Validation result: %s' % xmlschema.validate(etree.parse(file)))
        except etree.XMLSyntaxError as e:
            validation_errors.append(str(e))
        if len(validation_errors) > 0:
            print(validation_errors)

    def test_create_premis(self):

        premisgen = PremisGenerator(TestPremisCreation.tmp_ip_dir)
        premisgen.createPremis()
        self.validate_xml(TestPremisCreation.premis_file_path, TestPremisCreation.premis_schema_file)

    def test_add_premis_event(self):
        premisgen = PremisGenerator(TestPremisCreation.tmp_ip_dir_mig)

        premisinfo = {'outcome': 'success',
                      'task_name': 'SIPValidation',
                      'event_type': 'SIP validation',
                      'linked_object': 'this-is-a-package-id'}

        premisgen.addEvent('metadata/preservation/premis.xml', premisinfo)

        premis = '%s/metadata/preservation/premis.xml' % TestPremisCreation.tmp_ip_dir_mig

        self.validate_xml(premis, TestPremisCreation.premis_schema_file)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from lxml import objectify
from eatb.utils.XmlHelper import sequence_insert

from datetime import datetime

from lxml import etree, objectify

from eatb.utils.XmlHelper import q, XSI_NS

PREMIS_NS = 'http://www.loc.gov/premis/v3'
PREMIS_NSMAP = {None: PREMIS_NS}
P = objectify.ElementMaker(
    annotate=False,
    namespace=PREMIS_NS,
    nsmap=PREMIS_NSMAP)


class ParsedPremis:

    premis_successor_sections = ['object', 'event', 'agent', 'right']

    def __init__(self, f=None):
        if f is None:
            self.root = P.premis(
                {q(XSI_NS, 'schemaLocation'): PREMIS_NS + ' ../schemas/premis-3-0.xsd'},
            )
            self.root.set('version', '2.0')
        else:
            self.root = objectify.parse(f).getroot()

    def add_object(self, identifier_value):
        sequence_insert(
            self.root, P.object(
                {q(XSI_NS, 'type'): 'file'},
                P.objectIdentifier(
                    P.objectIdentifierType('LOCAL'),
                    P.objectIdentifierValue(identifier_value)
                ),
                P.objectCharacteristics(
                    P.compositionLevel(0),
                    P.format(
                        P.formatRegistry(
                            P.formatRegistryName(),
                            P.formatRegistryKey
                        )
                    )
                )
            ),
            self.premis_successor_sections
        )

    def add_event(self, identifier_value, outcome, linking_agent, linking_object=None):
        sequence_insert(
            self.root, P.event(
                P.eventIdentifier(
                    P.eventIdentifierType('LOCAL'),
                    P.eventIdentifierValue(identifier_value)
                ),
                P.eventType,
                P.eventDateTime(datetime.utcnow().isoformat()),
                P.eventOutcomeInformation(
                    P.eventOutcome(outcome)
                ),
                P.linkingAgentIdentifier(
                    P.linkingAgentIdentifierType('LOCAL'),
                    P.linkingAgentIdentifierValue(linking_agent)
                ),

                P.linkingAgentIdentifier(
                    P.linkingAgentIdentifierType('LOCAL'),
                    P.linkingAgentIdentifierValue(linking_object)
                )
                if linking_object is not None else None
            ),
            self.premis_successor_sections
        )

    def add_agent(self, identifier_value):
        sequence_insert(
            self.root, P.agent(
                P.agentIdentifier(
                    P.agentIdentifierType('LOCAL'),
                    P.agentIdentifierValue(identifier_value)
                ),
                P.agentName('E-ARK AIP to DIP Converter'),
                P.agentType('Software')
            ),
            self.premis_successor_sections
        )

    def __str__(self):
        return etree.tostring(self.root, encoding='UTF-8', pretty_print=True, xml_declaration=True)

    def to_string(self):
        return self.__str__()





#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import unittest

from eatb import ROOT
from eatb.metadata.parsed_mets import ParsedMets

test_dir = os.path.join(ROOT, 'tests/test_resources/metadata/mets/')
test_file = test_dir + 'METS_filesec.xml'


class TestParsedMets(unittest.TestCase):
    pmets = ParsedMets(test_dir)
    pmets.load_mets(test_file)
    def test_validate_files_size(self):
        """
        Must not validate if the files listed in the METS file does not match the actual file size
        """
        actual = self.pmets.mets_tree.getroot().tag
        self.assertEqual(actual, "{http://www.loc.gov/METS/}mets", "Root tag 'mets' not found")

    def test_get_file_elements(self):
        """
        File elements list must have one file element
        """
        file_elements = self.pmets.get_file_elements()
        self.assertIsNotNone(file_elements, "File elements must not be None")
        self.assertEqual(len(file_elements), 1, "File elements list must have one file element")

    def test_get_first_file_element(self):
        """
        Must return first file element
        """
        file_element = self.pmets.get_first_file_element()
        self.assertEqual(file_element.tag, "{http://www.loc.gov/METS/}file", "Must return a file element")

    def test_get_package_type(self):
        """
        Must return first file element
        """
        self.assertEqual("SIP", self.pmets.get_package_type())


if __name__ == '__main__':
    unittest.main()



import unittest

from eatb.cli import CliCommand, CliCommands


def gpg_encrypt_file_passphrase2(file, passphrase):
    return CliCommand.get_command("gpg_passphrase_encrypt_file", {'file': file, 'passphrase': passphrase})


def gpg_encrypt_file_passphrase(file, passphrase):
    """
    Encrypt file using passphrase
    :param file: file
    :param passphrase: passphrase
    :return: CLI command string
    """
    cli_commands = CliCommands()
    cli_command = CliCommand("gpg_passphrase_encrypt_file", cli_commands.get_command_template("gpg_passphrase_encrypt_file"))
    command = cli_command.get_command({'file': file, 'passphrase': passphrase})
    return command




class CliCommandTest(unittest.TestCase):
    def test_gpg_encrypt_file_passphrase(self):
        """
        Must return CLI command
        """
        cmd = gpg_encrypt_file_passphrase("test.txt", "12345")
        print(cmd)
        #out = subprocess32.call(cmd, cwd=os.path.join(root_dir,"testresources"))
        #self.assertEqual(0, out)
        # cmd = CliCommand.get("gpg_passphrase_decrypt_file",
        #                      {'encrypted_file': "test.txt.gpg", 'decrypted_file': "test_decrypted.txt", 'passphrase': "12345"})
        # out = subprocess32.call(cmd, cwd=os.path.join(root_dir,"testresources"))
        # self.assertEqual(0, out)
        # self.assertEqual("test", read_file_content(os.path.join(root_dir,"testresources/test_decrypted.txt")))
        # os.remove(os.path.join(root_dir,"testresources/test.txt.gpg"))
        # os.remove(os.path.join(root_dir,"testresources/test_decrypted.txt"))


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import unittest

from eatb import ROOT
from eatb.metadata.dip_parsed_premis import DIPPremis


class DIPPremisTest(unittest.TestCase):

    test_premis = os.path.join(ROOT, 'tests/test_resources/metadata/premis/premis.xml')

    def test_add_related_aips(self):
        """
        Test premis manipulation.
        """
        dip_premis = DIPPremis(DIPPremisTest.test_premis)
        dip_premis.add_related_aips(['d6e09e6a-5029-4273-99ef-f2307926379b',
                                     'd6e09e6a-5029-4273-99ef-f2307926379c'],
                                    'DIPAcquireAIPs')
        self.assertIsNotNone(dip_premis)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import json
import os
from subprocess import check_output

from eatb.cli import CliCommand, CliCommands


def get_hashed_filelist(strip_path_part, directory, commands=CliCommands()):
    """
    e.g. get_hashed_filelist("/home/user/", "/home/user/test")
    :param commands:
    :param strip_path_part: part of the path to be removed for the key
    :param directory: directory for which the hashed file list is to be created
    :return: hashed file list
    """
    cli_command = CliCommand("summainstdout", commands.get_command_template("summainstdout"))
    command = cli_command.get_command(dict(package_dir=directory))
    summain_out = check_output(command)
    json_summain_out = json.loads(summain_out)
    result = {}
    for entry in json_summain_out:
        if "SHA256" in entry:
            key = entry['Name'].lstrip(strip_path_part)
            result[key] = {"hash": entry['SHA256'], "path": entry["Name"]}
    return result





import re

from lxml import etree as ET


class ParsedDcat():
    """
    Parsed DCAT
    """
    dcat_tree = None
    dataset_xpath = "/rdf:RDF/dcat:Catalog/dcat:dataset/dcat:Dataset/"
    distribution_xpath = "%sdcat:distribution/dcat:Distribution" % dataset_xpath

    def __init__(self, xml_content):
        """
        Constructor takes root directory as argument; paths in EAD file are relative to this directory.

        @type       xml_content: string
        @param      xml_content: XML file content
        """
        self.ns = {'dcat': 'http://www.w3.org/ns/dcat#', 'foaf': 'http://xmlns.com/foaf/0.1/',
                   'rdf': 'http://www.w3.org/1999/02/22-rdf-syntax-ns#', 'owl': 'http://www.w3.org/2002/07/owl#',
                   'locn': 'http://www.w3.org/ns/locn#', 'hydra': 'http://www.w3.org/ns/hydra/core#',
                   'vcard': 'http://www.w3.org/2006/vcard/ns#', 'dmav': 'http://datamarket.at/2017/07/dmav/core#',
                   'dct': 'http://purl.org/dc/terms/'}
        self.dcat_tree = ET.XML(xml_content)

    def get_dataset_property_value(self, property):
        elms = self.dcat_tree.xpath("%s%s" % (self.dataset_xpath, property), namespaces=self.ns)
        if len(elms) == 1:
            return elms[0].text
        raise ValueError("Invalid metadata, data set property '%s' is required." % property)

    def get_dataset_property_values(self, properties):
        return {re.sub(r'[a-z]{1,10}:', '', prop).replace("/", "_"): self.get_dataset_property_value(prop) for prop in properties}

    def get_distribution_property_values(self):
        xpath_expr = "%s" % (self.distribution_xpath)
        elms = self.dcat_tree.xpath(xpath_expr, namespaces=self.ns)
        distr_props = {}
        for elm in elms:
            childs = list(elm.iter())
            dps = {}
            for child in childs:
                tag_name = re.sub(r'{.*}', '', child.tag)
                if tag_name == "license":
                    lic_hash = re.search(r'\/[0-9a-z]*\/$', child.attrib['{http://www.w3.org/1999/02/22-rdf-syntax-ns#}resource']).group(0)[1:-1]
                    dps[tag_name] = lic_hash
                else:
                    dps[tag_name] = child.text.strip() if child.text else ""
            del dps['Distribution']
            distr_props[dps['title']] = dps
        return distr_props



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import unittest

from lxml import etree

import eatb.metadata.premis
from eatb import ROOT
from eatb.metadata.parsed_premis import ParsedPremis
from eatb.utils.xmlutils import pretty_xml_string
from eatb.csip_validation import XmlValidation

premis_skeleton = os.path.join(ROOT, 'tests/test_resources/metadata/premis/PREMIS_skeleton.xml')
premis_schema_file = os.path.join(ROOT, 'tests/test_resources/schemas/premis-v3-0.xsd')


class TestPremisManipulate(unittest.TestCase):


    def test_premis_manipulate(self):
        with open(os.path.join(ROOT, premis_skeleton), 'r') as premis_file:
            self.my_premis = ParsedPremis(premis_file)
            self.my_premis.add_agent('Aip2Dip')
            self.my_premis.add_event('Migration01', 'success', 'Aip2Dip',
                                           'linking_agent')
            self.my_premis.add_object('file.txt')
            premis_xml = pretty_xml_string(self.my_premis.to_string())
            print(premis_xml)
            xmlval = XmlValidation()
            parsed_xml = etree.fromstring(premis_xml)
            parsed_schema = etree.parse(premis_schema_file)
            validation_result = xmlval.validate_XML(parsed_xml,parsed_schema)
            print(validation_result)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import unittest

from eatb import ROOT
from eatb.metadata.parsed_ead import ParsedEad


class TestParsedEad(unittest.TestCase):
    test_dir = os.path.join(ROOT, 'tests/test_resources/EAD-test/')

    def test_get_root_element(self):
        """
        Must not validate if the files listed in the EAD file does not match the actual file size
        """
        pead = ParsedEad(TestParsedEad.test_dir, TestParsedEad.test_dir + 'metadata/descriptive/EAD-example1.xml')
        actual = pead.ead_tree.getroot().tag
        self.assertEqual(actual, "{http://ead3.archivists.org/schema/}ead", "Root tag 'ead' not found")

    def test_get_dao_elements(self):
        """
        Test get dao elements
        """
        pead = ParsedEad(TestParsedEad.test_dir, TestParsedEad.test_dir + 'metadata/descriptive/EAD-example1.xml')
        dao_elements = pead.get_dao_elements()
        self.assertIsNotNone(dao_elements, "DAO elements must not be None")
        self.assertEqual(len(dao_elements), 2, "DAO elements list must have two DAO element")

    def test_first_metadata_value_in_ancestry_path(self):
        """
        Test get closest unittitle element value (c04)
        """
        pead = ParsedEad(TestParsedEad.test_dir, TestParsedEad.test_dir + 'metadata/descriptive/EAD-example1.xml')
        dao_elements = pead.get_dao_elements()
        for dao_elm in dao_elements:
            self.assertEqual("Record - Adams-Ayers", pead._first_md_val_ancpath(dao_elm, "unittitle"))

    def test_first_metadata_value_in_ancestry_path_unitdatestructured(self):
        """
        Test get closest unittitle element value (c04)
        """
        pead = ParsedEad(TestParsedEad.test_dir, TestParsedEad.test_dir + 'metadata/descriptive/EAD-example1.xml')
        dao_elements = pead.get_dao_elements()
        for dao_elm in dao_elements:
            self.assertEqual("22.04.2016", pead._first_md_val_ancpath(dao_elm, "unitdatestructured", "ead:datesingle"))

    def test_first_metadata_value_in_ancestry_path_unitdatestructured_range(self):
        """
        Test get closest unittitle element value (c04)
        """
        pead = ParsedEad(TestParsedEad.test_dir, TestParsedEad.test_dir + 'metadata/descriptive/EAD-example6.xml')
        dao_elements = pead.get_dao_elements()
        for dao_elm in dao_elements:
            self.assertEqual("22.04.2016", pead._first_md_val_ancpath(dao_elm, "unitdatestructured", "ead:daterange/ead:fromdate"))
            self.assertEqual("28.04.2016", pead._first_md_val_ancpath(dao_elm, "unitdatestructured", "ead:daterange/ead:todate"))

    def test_first_metadata_value_in_ancestry_path_origination(self):
        """
        Test get closest unittitle element value (c04)
        """
        pead = ParsedEad(TestParsedEad.test_dir, TestParsedEad.test_dir + 'metadata/descriptive/EAD-example5.xml')
        dao_elements = pead.get_dao_elements()
        for dao_elm in dao_elements:
            self.assertEqual("Test Agency", pead._first_md_val_ancpath(dao_elm, "origination", "ead:corpname/ead:part"))

    def test_first_metadata_value_in_ancestry_path_origination_xpath(self):
        """
        Test get closest unittitle element value (text access xpath)
        """
        pead = ParsedEad(TestParsedEad.test_dir, TestParsedEad.test_dir + 'metadata/descriptive/EAD-example5.xml')
        dao_elements = pead.get_dao_elements()
        for dao_elm in dao_elements:
            self.assertEqual("Test Agency", pead._first_md_val_ancpath(dao_elm, "origination", text_accessor="*/ead:part"))

    def test_first_metadata_value_in_ancestry_path_c03(self):
        """
        Test get closest unittitle element value (c03)
        """
        pead = ParsedEad(TestParsedEad.test_dir, TestParsedEad.test_dir + 'metadata/descriptive/EAD-example2.xml')
        dao_elements = pead.get_dao_elements()
        for dao_elm in dao_elements:
            self.assertEqual("Adams-Ayers", pead._first_md_val_ancpath(dao_elm, "unittitle"))

    def test_first_metadata_value_in_ancestry_path_c02(self):
        """
        Test get closest unittitle element value (c02)
        """
        pead = ParsedEad(TestParsedEad.test_dir, TestParsedEad.test_dir + 'metadata/descriptive/EAD-example3.xml')
        dao_elements = pead.get_dao_elements()
        for dao_elm in dao_elements:
            self.assertEqual("Incoming Correspondence", pead._first_md_val_ancpath(dao_elm, "unittitle"))

    def test_first_metadata_value_in_ancestry_path_c01(self):
        """
        Test get closest unittitle element value (c01)
        """
        pead = ParsedEad(TestParsedEad.test_dir, TestParsedEad.test_dir + 'metadata/descriptive/EAD-example4.xml')
        dao_elements = pead.get_dao_elements()
        for dao_elm in dao_elements:
            self.assertEqual("Correspondence", pead._first_md_val_ancpath(dao_elm, "unittitle"))

    def test_dao_title_tuples(self):
        """
        Test dao tuples
        """
        parsed_ead = TestParsedEad.test_dir
        ead_file_path = TestParsedEad.test_dir + 'metadata/descriptive/EAD-example1.xml'
        pead = ParsedEad(parsed_ead, ead_file_path)
        md_tag = "unittitle"
        res = pead.dao_path_mdval_tuples(md_tag)
        self.assertEqual("representations/rep1/data/Example1.docx", res[0]['path'])
        self.assertEqual("representations/rep2/data/Example1.pdf", res[1]['path'])
        self.assertEqual("Record - Adams-Ayers", res[0]['mdvalue'])
        self.assertEqual("Record - Adams-Ayers", res[1]['mdvalue'])

    def test_dao_unitdatestructured_datevalue_subelement(self):
        """
        Test dao tuples
        """
        parsed_ead = TestParsedEad.test_dir
        ead_file_path = TestParsedEad.test_dir + 'metadata/descriptive/EAD-example1.xml'
        pead = ParsedEad(parsed_ead, ead_file_path)
        md_tag = "unitdatestructured"
        res = pead.dao_path_mdval_tuples(md_tag, "ead:datesingle")
        self.assertEqual("22.04.2016", res[0]['mdvalue'])

    def test_c_level(self):
        """
        Test get closest unittitle element value (c01)
        """
        pead = ParsedEad(TestParsedEad.test_dir, TestParsedEad.test_dir + 'metadata/descriptive/EAD-example1.xml')
        dao_elements = pead.get_dao_elements()
        for dao_elm in dao_elements:
            self.assertEqual("item", pead._first_md_val_ancpath(dao_elm, "[Cc][0,1][0-9]", "level", True))

    def test_dao_clevel_attribute_value(self):
        """
        Test dao tuples
        """
        parsed_ead = TestParsedEad.test_dir
        ead_file_path = TestParsedEad.test_dir + 'metadata/descriptive/EAD-example1.xml'
        pead = ParsedEad(parsed_ead, ead_file_path)
        md_tag = "[Cc][0,1][0-9]"
        res = pead.dao_path_mdval_tuples(md_tag, "level", True)
        self.assertEqual("item", res[0]['mdvalue'])



if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import unittest

from eatb.utils.stringutils import whitespace_separated_text_to_dict


class TestStringUtils(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        pass

    @classmethod
    def tearDownClass(cls):
        pass

    def test_whitespace_separated_text_to_dict(self):
        input_text_map = """
        IP.AVID.RA.18005.rep0.seg0 4294e5eb-6d95-4b31-b544-53eb50711a56
        IP.AVID.RA.18005.rep1.seg1 d9bb0f20-aa8d-43b8-95ca-9bfcb23da832
        """
        expected_result_map = {
            'IP.AVID.RA.18005.rep0.seg0': '4294e5eb-6d95-4b31-b544-53eb50711a56',
            'IP.AVID.RA.18005.rep1.seg1': 'd9bb0f20-aa8d-43b8-95ca-9bfcb23da832',
        }
        result_map = whitespace_separated_text_to_dict(input_text_map)
        print(result_map)
        self.assertEqual(2, len(result_map), "Incorrect size of result map")
        self.assertEqual('4294e5eb-6d95-4b31-b544-53eb50711a56', expected_result_map.pop('IP.AVID.RA.18005.rep0.seg0'))
        self.assertEqual('d9bb0f20-aa8d-43b8-95ca-9bfcb23da832', expected_result_map.pop('IP.AVID.RA.18005.rep1.seg1'))

    def test_whitespace_in_package_name(self):
        input_text_map = """
        HU_MNL_OL_XX - IDfd20a7fe-ab32-4bdb-ba7c-b10d9449025a    urn:uuid:8e231c2d-ad35-b73f-5923-f0d9aa8a9e24
        HU_MNL_OL_XX - ID1ac7249b-9e16-4d27-a0fc-0d8459e5066b    urn:uuid:5dae14ae-00aa-6ecc-3008-0e8ac2e692c4
        """
        expected_result_map = {
            'HU_MNL_OL_XX - IDfd20a7fe-ab32-4bdb-ba7c-b10d9449025a': 'urn:uuid:8e231c2d-ad35-b73f-5923-f0d9aa8a9e24',
            'HU_MNL_OL_XX - ID1ac7249b-9e16-4d27-a0fc-0d8459e5066b': 'urn:uuid:5dae14ae-00aa-6ecc-3008-0e8ac2e692c4',
        }
        result_map = whitespace_separated_text_to_dict(input_text_map)
        self.assertEqual(2, len(result_map), "Incorrect size of result map")
        self.assertEqual('urn:uuid:8e231c2d-ad35-b73f-5923-f0d9aa8a9e24', expected_result_map.pop('HU_MNL_OL_XX - IDfd20a7fe-ab32-4bdb-ba7c-b10d9449025a'))
        self.assertEqual('urn:uuid:5dae14ae-00aa-6ecc-3008-0e8ac2e692c4', expected_result_map.pop('HU_MNL_OL_XX - ID1ac7249b-9e16-4d27-a0fc-0d8459e5066b'))

    def test_tab_whitespace_separated_text_to_dict(self):
        input_text_map = """
        IP.AVID.RA.18005.rep0.seg0  \t 4294e5eb-6d95-4b31-b544-53eb50711a56
        """
        expected_result_map = {
            'IP.AVID.RA.18005.rep0.seg0': '4294e5eb-6d95-4b31-b544-53eb50711a56',
        }
        result_map = whitespace_separated_text_to_dict(input_text_map)
        self.assertEqual(1, len(result_map), "Incorrect size of result map")
        self.assertEqual('4294e5eb-6d95-4b31-b544-53eb50711a56', expected_result_map.pop('IP.AVID.RA.18005.rep0.seg0'))


if __name__ == '__main__':
    unittest.main()
